<?php
/**
* Database auth plug-in for phpBB3
*
* Authentication plug-ins is largely down to Sergey Kanareykin, our thanks to him.
*
* This is for authentication for PAELLOS
*
* @package login
* @version $Id$
* @copyright (c) 2008 Observatoire de Paris
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

include_once($phpbb_root_path . 'includes/db/mysql.' . $phpEx);

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
    exit;
}

/**
* Login function
*/
function login_paellos(&$username, &$password)
{
    global $db, $config;

    // do not allow empty password
    if (!$password)
    {
        return array(
            'status'    => LOGIN_ERROR_PASSWORD,
            'error_msg'    => 'NO_PASSWORD_SUPPLIED',
            'user_row'    => array('user_id' => ANONYMOUS),
        );
    }

    if (!$username)
    {
        return array(
            'status'    => LOGIN_ERROR_USERNAME,
            'error_msg'    => 'LOGIN_ERROR_USERNAME',
            'user_row'    => array('user_id' => ANONYMOUS),
        );
    }
    
    //if ($username == "admin") {
    //    include_once('auth_db.php');
    //    return(login_db($username, $password));
    //}
    
    $paellos_db = new dbal_mysql();
    if (is_array($paellos_db->sql_connect($config['paellos_server'], $config['paellos_dbuser'], $config['paellos_dbpassword'], $config['paellos_database']))) {
        //$db_error = $paellos_db->sql_error();
        return array(
            'status'    => LOGIN_ERROR_EXTERNAL_AUTH,
            'error_msg'    => 'ERR_CONNECTING_SERVER',
            'user_row'    => array('user_id' => ANONYMOUS),
        );
    }
    
    $sql = "SELECT numero, username, password, email, fonction, datefermeture, formation, valide, prenom, nom " .
        "FROM utilisateurs WHERE username = '" . $db->sql_escape($username) . "'";
    $result = $paellos_db->sql_query($sql);
    $row = $paellos_db->sql_fetchrow($result);
    $paellos_db->sql_freeresult($result);
    $paellos_db->sql_close();
    
    if (!$row)
    {
        return array(
            'status'    => LOGIN_ERROR_USERNAME,
            'error_msg'    => 'LOGIN_ERROR_USERNAME',
            'user_row'    => array('user_id' => ANONYMOUS),
        );
    }
    
    // test mot de passe
    if (strlen($row['password']) < 20) {
        $salt = substr($row['password'], 0, 2);
        $mauvais = ($row['password'] != crypt($password, $salt));
    } else {
        // nouvelle méthode
        // SHA-256 (32 octets) avec salt 64 bits (8 octets)
        // -> 80 caractères avec un encodage hexa
        // requiert PHP 5 >= 5.1.2
        $salt = pack('H*', substr($row['password'], 0, 16));
        $enc = substr($row['password'], 16, 64);
        $ctx = hash_init('sha256');
        hash_update($ctx, $salt);
        hash_update($ctx, $password);
        $mauvais = ($enc != hash_final($ctx));
    }
    if ($mauvais) {
        return array(
            'status'        => LOGIN_ERROR_PASSWORD,
            'error_msg'        => 'LOGIN_ERROR_PASSWORD',
            'user_row'        => $row,
        );
    }
    
    // User inactive...
    if ($row['valide'] != '1' || $row['datefermeture'] != '')
    {
        return array(
            'status'        => LOGIN_ERROR_ACTIVE,
            'error_msg'        => 'ACTIVE_ERROR',
            'user_row'        => $row,
        );
    }
    
    $fonction = $row['fonction'];
    $email = $row['email'];
    $formation = $row['formation'];
    $prenom = $row['prenom'];
    $nom = $row['nom'];
    
    $sql ='SELECT user_id, username, user_password, user_passchg, user_email, user_type FROM ' .
        USERS_TABLE . " WHERE username_clean = '" . $db->sql_escape(utf8_clean_string($username)) . "'";
    $result = $db->sql_query($sql);
    $row = $db->sql_fetchrow($result);
    $db->sql_freeresult($result);

    if ($row)
    {
        // l'utilisateur est deja dans la base phpBB
           
        // User inactive...
        if ($row['user_type'] == USER_INACTIVE || $row['user_type'] == USER_IGNORE)
        {
            return array(
                'status'        => LOGIN_ERROR_ACTIVE,
                'error_msg'        => 'ACTIVE_ERROR',
                'user_row'        => $row,
            );
        }

        // Successful login...
        return array(
            'status'        => LOGIN_SUCCESS,
            'error_msg'        => false,
            'user_row'        => $row,
        );
    }
    else
    {
        // il faut ajouter l'utilisateur dans la base phpBB
        
        // retrieve default group id
        if ($fonction == 'étudiant')
            $groupe = 'Etudiants ' . $formation;
        else if ($fonction == 'tuteur' || $fonction == 'supertuteur')
            $groupe = 'Tuteurs ' . $formation;
        else if ($fonction == 'système' || $fonction == 'administrateur')
            $groupe = 'ADMINISTRATORS';
        else
            $groupe = 'GUESTS';
        $sql = 'SELECT group_id FROM ' . GROUPS_TABLE . " WHERE group_name = '" . $db->sql_escape($groupe) . "'";
        $result = $db->sql_query($sql);
        $row = $db->sql_fetchrow($result);
        $db->sql_freeresult($result);

        if (!$row)
        {
            trigger_error('NO_GROUP');
        }

        // generate user account data
        $ldap_user_row = array(
            'username'        => $username,
            'user_password'    => phpbb_hash($password),
            'user_email'    => $email,
            'group_id'        => (int) $row['group_id'],
            'user_type'        => USER_NORMAL
        );
        $cp_data = array(
            'pf_firstname'        => $prenom,
            'pf_lastname'        => $nom
        );
        // this is the user's first login so create an empty profile
        return array(
            'status'        => LOGIN_SUCCESS_CREATE_PROFILE,
            'error_msg'        => false,
            'user_row'        => $ldap_user_row,
            'cp_data'       => $cp_data
        );
    }
        
}

/**
* This function is used to output any required fields in the authentication
* admin panel. It also defines any required configuration table fields.
*/
function acp_paellos(&$new)
{
    global $user;

    $chemin_sites = $new['chemin_sites'];
    if ($chemin_sites == '') {
        $host = (!empty($_SERVER['HTTP_HOST'])) ? (string) strtolower($_SERVER['HTTP_HOST']) : ((!empty($_SERVER['SERVER_NAME'])) ? $_SERVER['SERVER_NAME'] : getenv('SERVER_NAME'));
        $chemin_sites = 'http://'.$host.'/';
    }
    $tpl = '

    <dl>
        <dt><label for="paellos_server">Serveur PAELLOS:</label><br /><span>nom de la machine serveur PAELLOS</span></dt>
        <dd><input type="text" id="paellos_server" size="40" name="config[paellos_server]" value="' . $new['paellos_server'] . '" /></dd>
    </dl>
    <dl>
        <dt><label for="paellos_dbuser">Administrateur PAELLOS:</label><br /><span>Nom à utiliser pour se connecter à la base de données PAELLOS</span></dt>
        <dd><input type="text" id="paellos_dbuser" size="40" name="config[paellos_dbuser]" value="' . $new['paellos_dbuser'] . '" /></dd>
    </dl>
    <dl>
        <dt><label for="paellos_dbpassword">Mot de passe PAELLOS:</label><br /><span>Mot de passe de l'."'".'administrateur PAELLOS</span></dt>
        <dd><input type="password" id="paellos_dbpassword" size="40" name="config[paellos_dbpassword]" value="' . $new['paellos_dbpassword'] . '" /></dd>
    </dl>
    <dl>
        <dt><label for="paellos_database">Base de données PAELLOS:</label><br /><span>Nom de la base de données PAELLOS</span></dt>
        <dd><input type="text" id="paellos_database" size="40" name="config[paellos_database]" value="' . $new['paellos_database'] . '" /></dd>
    </dl>
    <dl>
        <dt><label for="chemin_sites">Adresse des sites:</label><br /><span>Chemin des sites de formation</span></dt>
        <dd><input type="text" id="chemin_sites" size="40" name="config[chemin_sites]" value="' . $chemin_sites . '" /></dd>
    </dl>
    ';

    // These are fields required in the config table
    return array(
        'tpl'        => $tpl,
        'config'    => array('paellos_server', 'paellos_dbuser', 'paellos_dbpassword', 'paellos_database', 'chemin_sites')
    );
}

/**
* Autologin function
*
* @return array containing the user row or empty if no auto login should take place
*/
function autologin_paellos()
{
    global $db, $config;
    
    if (isset($_COOKIE["site"]))
        $nom_site = $_COOKIE["site"];
    else
        return array();
    $url_username = $config['chemin_sites'] . "/" . $nom_site . "/session.jsp";
    $crl = curl_init();
    curl_setopt ($crl, CURLOPT_URL, $url_username);
    curl_setopt ($crl, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt ($crl, CURLOPT_CONNECTTIMEOUT, 5);
    curl_setopt ($crl, CURLOPT_FAILONERROR, TRUE);
    if (isset($_COOKIE["JSESSIONID"]))
        curl_setopt ($crl, CURLOPT_COOKIE, 'JSESSIONID='.$_COOKIE["JSESSIONID"] );
    $res = curl_exec($crl);
    if ($res === FALSE) {
        error_log('autologin_paellos : '.$url_username.' : '.curl_error($crl));
        return array();
    }
    $username = trim($res);
    curl_close($crl);
    setcookie('JSESSIONID', '', 1, '/');
    if (! $username || $username == '')
        return array();
    
    $sql = 'SELECT * FROM ' . USERS_TABLE . "
        WHERE username_clean = '" . $db->sql_escape(utf8_clean_string($username)) . "'";
    $result = $db->sql_query($sql);
    $row = $db->sql_fetchrow($result);
    $db->sql_freeresult($result);
    
    if ($row) {
        return ($row['user_type'] == USER_INACTIVE || $row['user_type'] == USER_IGNORE) ? array() : $row;
    }
    if (!function_exists('user_add'))
    {
        global $phpbb_root_path, $phpEx;

        include($phpbb_root_path . 'includes/functions_user.' . $phpEx);
    }

    // create the user if he does not exist yet
    user_add(user_row_paellos($username, ""), get_cp_data($username));

    $sql = 'SELECT * FROM ' . USERS_TABLE . "
        WHERE username_clean = '" . $db->sql_escape(utf8_clean_string($username)) . "'";
    $result = $db->sql_query($sql);
    $row = $db->sql_fetchrow($result);
    $db->sql_freeresult($result);

    if ($row)
    {
        return $row;
    }
    return array();
}

/**
* This function generates an array which can be passed to the user_add function in order to create a user
*/
function user_row_paellos($username, $password)
{
    global $db, $config, $user;
    
    $paellos_db = new dbal_mysql();
    // sql_connect(sqlserver, sqluser, sqlpassword, database, bool port, bool persistency, bool new_link)
    if (is_array($paellos_db->sql_connect($config['paellos_server'], $config['paellos_dbuser'], $config['paellos_dbpassword'], $config['paellos_database']))) {
        trigger_error('ERR_CONNECTING_SERVER');
    }
    
    $sql = "SELECT numero, username, password, email, fonction, datefermeture, formation, valide, prenom, nom " .
        "FROM utilisateurs WHERE username = '" . $db->sql_escape($username) . "'";
    $result = $paellos_db->sql_query($sql);
    $row = $paellos_db->sql_fetchrow($result);
    $paellos_db->sql_freeresult($result);
    $paellos_db->sql_close();
    
    if (!$row)
    {
        trigger_error('NO_GROUP');
    }
    
    $fonction = $row['fonction'];
    $email = $row['email'];
    $formation = $row['formation'];
    $prenom = $row['prenom'];
    $nom = $row['nom'];
    if ($fonction == 'étudiant')
        $groupe = 'Etudiants ' . $formation;
    else if ($fonction == 'tuteur' || $fonction == 'supertuteur')
        $groupe = 'Tuteurs ' . $formation;
    else if ($fonction == 'système' || $fonction == 'administrateur')
        $groupe = 'ADMINISTRATORS';
    else
        $groupe = 'GUESTS';
    
    $sql = 'SELECT group_id FROM ' . GROUPS_TABLE . " WHERE group_name = '" . $db->sql_escape($groupe) . "'";
    $result = $db->sql_query($sql);
    $row = $db->sql_fetchrow($result);
    $db->sql_freeresult($result);

    if (!$row)
    {
        trigger_error('NO_GROUP');
    }
    // generate user account data
    return array(
        'username'        => $username,
        'user_password'    => phpbb_hash($password),
        'user_email'    => $email,
        'group_id'        => (int) $row['group_id'],
        'user_type'        => USER_NORMAL,
        'user_ip'        => $user->ip
    );
}

function get_cp_data($username)
{
    global $db, $config;
    
    $paellos_db = new dbal_mysql();
    if (is_array($paellos_db->sql_connect($config['paellos_server'], $config['paellos_dbuser'], $config['paellos_dbpassword'], $config['paellos_database']))) {
        trigger_error('ERR_CONNECTING_SERVER');
    }
    
    $sql = "SELECT prenom, nom FROM utilisateurs WHERE username = '" . $db->sql_escape($username) . "'";
    $result = $paellos_db->sql_query($sql);
    $row = $paellos_db->sql_fetchrow($result);
    $paellos_db->sql_freeresult($result);
    $paellos_db->sql_close();
    
    if (!$row)
    {
        trigger_error('NO_GROUP');
    }
    
    $prenom = $row['prenom'];
    $nom = $row['nom'];
    return array(
        'pf_firstname'       => $prenom,
        'pf_lastname'        => $nom
    );
}

/**
* The session validation function checks whether the user is still logged in
*
* @return boolean true if the given user is authenticated or false if the session should be closed
*/
function validate_session_paellos(&$user)
{
    return !isset($_COOKIE["JSESSIONID"]);
}

/*
function logout_paellos(&$user, &$new_session)
{
    setcookie('JSESSIONID', '', 1, '/');
    unset($_COOKIE["JSESSIONID"]);
}
*/

?>
