CKEDITOR.plugins.add('equation', {
    // remarque : il faut définir config.serveur_equations dans config.js
    lang: ['fr', 'en'],
    init: function(editor) {
        var nomPlugin = 'equation';
        CKEDITOR.dialog.add(nomPlugin, function(editor) {
            return {
                title : editor.lang.equation.titre,
                minWidth : 500,
                minHeight : 300,
                buttons: [CKEDITOR.dialog.okButton, CKEDITOR.dialog.cancelButton],
                onLoad: function() { },
                onShow: function() {
                    var sel, element;
                    sel = editor.getSelection();
                    element = sel.getStartElement();
                    if (element)
                        element = element.getAscendant('img', true );
                    
                    if (!element || element.getName() != 'img' || element.data( 'cke-realelement' )) {
                        element = editor.document.createElement('img');
                        element.setAttribute('class', 'equation');
                        this.insertMode = true;
                    } else
                        this.insertMode = false;
                    
                    this.element = element;
                    this.setupContent(this.element);
                },
                onOk: function() {
                    var dialog, img;
                    dialog = this;
                    img = this.element;
                    if (this.insertMode)
                        editor.insertElement(img);
                    this.commitContent(img);
                },
                onHide: function() { },
                onCancel: function() { },
                resizable: CKEDITOR.DIALOG_RESIZE_BOTH,
                contents: [{
                    id: 'info',
                    label: 'Informations',
                    elements: [
                    {
                        type: 'textarea',
                        id: 'texte',
                        label: editor.lang.equation.texte,
                        title: editor.lang.equation.info,
                        labelLayout: 'horizontal',
                        widths: ['30%','70%'],
                        rows: 3,
                        validate : CKEDITOR.dialog.validate.notEmpty(editor.lang.equation.vide),
                        setup: function(element) {
                            var src, ind, texte;
                            src = element.getAttribute('src');
                            if (src != null && src != '') {
                                ind = src.indexOf('equation=');
                                if (ind != -1)
                                    texte = decodeURIComponent(src.substring(ind + 9).replace(/\+/g,' '));
                                else
                                    texte = '';
                                this.setValue(texte);
                            }
                        },
                        commit : function(element) {
                            var texte;
                            texte = encodeURIComponent(this.getValue());
                            element.setAttribute('src', editor.config.serveur_equations + texte);
                            if (element.getAttribute('width') != null)
                                element.removeAttribute('width');
                            if (element.getAttribute('height') != null)
                                element.removeAttribute('height');
                        }
                    },
                    {
                        type: 'html',
                        id: 'syntaxe',
                        label: editor.lang.equation.syntaxe,
                        html : '<div style="text-align: right"><a href="' + CKEDITOR.plugins.getPath('equation') +
                            'doc_syntaxe/syntaxe_' + editor.langCode +
                            '.html" target="_blank" style="text-decoration: underline; cursor: pointer; color: #0000FF">' +
                            editor.lang.equation.syntaxe + '</a></div>'
                    },
                    {
                        type: 'button',
                        id: 'apercu',
                        label: editor.lang.equation.apercu,
                        onClick : function(element) {
                            var dialog, champ_texte, champ_image, src, div, img;
                            dialog = this.getDialog();
                            champ_texte = dialog.getContentElement('info', 'texte');
                            champ_image = dialog.getContentElement('info', 'image');
                            src = editor.config.serveur_equations + encodeURIComponent(champ_texte.getValue());
                            div = champ_image.getElement();
                            if (div.getFirst() == null) {
                                img = div.getDocument().createElement('img', {'attributes': {'src': src}});
                                div.append(img);
                            } else {
                                div.getFirst().setAttribute('src', src);
                            }
                        }
                    },
                    {
                        type: 'html',
                        id: 'image',
                        html : '<div></div>',
                        onShow : function() {
                            var div;
                            div = this.getElement();
                            if (div != null && div.getFirst() != null)
                                div.getFirst().remove();
                        }
                    }
                    ]
                }]
            }
        });
        editor.addCss('img.equation { vertical-align: middle }');
        editor.keystrokeHandler.keystrokes[CKEDITOR.CTRL + 69 /* E */] = 'equation';
        editor.addCommand(nomPlugin, new CKEDITOR.dialogCommand(nomPlugin));
        editor.ui.addButton('Equation', {
            label: editor.lang.equation.info,
            command: nomPlugin,
            icon: CKEDITOR.plugins.getPath('equation') + 'equation.png'
        } );
        editor.on('doubleclick', function(evt) {
            var element = evt.data.element;
            if ( element.is( 'img' ) && !element.data( 'cke-realelement' ) && !element.isReadOnly() )
                evt.data.dialog = 'equation';
        });
        if (editor.contextMenu) {
            editor.addMenuGroup('myGroup');
            editor.addMenuItem('itemEquation', {
                label : 'Modifier',
                command : nomPlugin,
                group : 'myGroup'
            });
            editor.contextMenu.addListener( function(element) {
                if (element)
                    element = element.getAscendant( 'img', true );
                if (element && !element.isReadOnly() && !element.data('cke-realelement'))
                    return { itemEquation : CKEDITOR.TRISTATE_OFF };
                return null;
            });
        }
    }
    
} );
