/*
PAELLOS - Plateforme d'Apprentissage En Ligne Libre Orientée Sémantique

Copyright (C) 2011 Observatoire de Paris

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package paellos;

import java.io.*;
import java.security.NoSuchAlgorithmException;
import java.util.*;

import java.text.ParseException;
import java.text.SimpleDateFormat;

import javax.servlet.*;
import javax.servlet.http.*;

import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.SourceLocator;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import org.apache.xml.utils.SAXSourceLocator;
import org.apache.xml.utils.WrappedRuntimeException;

import paellos.jdb.*;
import paellos.base.*;

/**
 * Servlet pour CELA: administration
 *
 * @author Damien Guillaume
 */
public class Administration extends HttpServlet {
    
    // valeurs par défaut, qu'on change dans le fichier ServletProperties
    String baseSiteEtudiants;
    String baseSiteTuteurs;
    protected String URLServeur;
    protected String pageLogin;
	protected String smtpServer;
	protected String emailAdmin;
    protected String userDBURL;
    protected String userDBUSER;
    protected String userDBPASSWORD;
    protected String[] noms_formations;
    protected String[] titres_formations;
    protected String indexLucene;
    protected String cheminXMLSite;
    protected String baseSitePublic;
    static final String[] fonctionval = {"étudiant", "tuteur", "supertuteur", "auteur", "administrateur", "système"}; // valeurs dans la bdd
    static final String[] fonctionstr = {"etudiant", "tuteur", "supertuteur", "auteur", "administrateur", "systeme"}; // clés pour LocalStrings
    
    // à faire: mettre ces valeurs dans ServletProperties
    static final String[] connectionval = {"", "modem", "cable", "ADSL", "Renater", "autre"};
    static final String[] accesval = {"", "domicile", "école", "université", "autre"};
    static final String[] niveauval = {"", "scientifique", "non scientifique"};
    
    UserDB userDB = null;
    CoordDB coordDB = null;
    ConnectionDB connDB = null;
    LectureDB lectDB = null;
    ExerciceDB exoDB = null;
    
    
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
        Properties props = new Properties();
        try {
            props.load(new InputStreamReader(getClass().getResourceAsStream("ServletProperties"), "UTF-8"));
            baseSiteEtudiants = props.getProperty("baseSiteEtudiants");
            baseSiteTuteurs = props.getProperty("baseSiteTuteurs");
            URLServeur = props.getProperty("URLServeur");
            pageLogin = props.getProperty("pageLogin");
            smtpServer = props.getProperty("smtpServer");
            emailAdmin = props.getProperty("emailAdmin");
            userDBURL = props.getProperty("userDBURL");
            userDBUSER = props.getProperty("userDBUSER");
            userDBPASSWORD = props.getProperty("userDBPASSWORD");
            String s_formations = props.getProperty("formations");
            if (s_formations == null || "".equals(s_formations)) {
                noms_formations = null;
                titres_formations = null;
            } else {
                String[] formations = s_formations.split("\\|");
                noms_formations = new String[formations.length];
                titres_formations = new String[formations.length];
                for (int i=0; i<formations.length; i++) {
                    String[] nom_titre = formations[i].split(":");
                    noms_formations[i] = nom_titre[0];
                    if (nom_titre.length > 0)
                        titres_formations[i] = nom_titre[1];
                    else
                        titres_formations[i] = noms_formations[i];
                }
            }
            indexLucene = props.getProperty("indexLucene");
            cheminXMLSite = props.getProperty("cheminXMLSite");
            baseSitePublic = props.getProperty("baseSitePublic");
        } catch (IOException ex) {
            log(Strings.get("erreur_proprietes") + " " + ex.getMessage());
        }
        userDB = new UserDB(userDBURL, userDBUSER, userDBPASSWORD);
        try {
            if (!userDB.tableExists()) {
                log(Strings.get("info_creation_table_utilisateurs"));
                userDB.createDB();
            }
        } catch (JDBException ex) {
            log("Erreur: création de userDB : " + ex.getMessage());
            return;
        }
        coordDB = new CoordDB(userDBURL, userDBUSER, userDBPASSWORD);
        try {
            if (!coordDB.tableExists()) {
                log(Strings.get("info_creation_table_coordonnees"));
                coordDB.createDB();
            }
        } catch (JDBException ex) {
            log("Erreur: création de coordDB : " + ex.getMessage());
            return;
        }
    }
    
    /**
     * Paramètres possibles pour une requête POST :
     * login : login pour la connexion au site
     * password : mot de passe pour la connexion au site
     * action : modifier_util|modifier_coord|fermer|supprimer_util
     * si modifier_util: numero, username, nom, prenom, password, email, fonction, naissance, formation, niveau, modules, matiere, classe,
     * dateinscription, datefermeture, numtuteur, centre, valide
     * si modifier_coord: numero, numuser, rue1, rue2, codepostal, ville, pays, telephone, connection, acces
     * si fermer: comptes, tri (pour revenir à la liste)
     * si supprimer_util: page, comptes, tri (pour revenir à la liste)
     */
    public void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        if (req.getCharacterEncoding() == null)
            req.setCharacterEncoding("UTF-8");
        if (req.getParameter("login") != null) {
            final HttpSession session = req.getSession(true);
            String nom = req.getParameter("login");
            String password = req.getParameter("password");
            if(nom == null || nom.length() == 0)
                nom = "anonymous";
            
            // recherche de l'utilisateur dans la base
            User user = null;
            final User finduser = new User(nom);
            finduser.valide = "1";
            try {
                Vector<User> v = userDB.findUsers(finduser);
                if (v.size() > 0)
                    user = v.get(0); // on prend le premier utilisateur avec le username
            } catch (JDBException ex) {
                log("JDBException: " + ex.getMessage());
            }
            if (user == null) {
                log(Strings.get("utilisateur_non_trouve") + " " + nom + " -> " + Strings.get("acces_refuse"));
                envoyerErreur(resp, Strings.get("acces_refuse"));
                return;
            }
            
            // test mot de passe
            try {
                if (password == null || !JDB.testEnc(password, user.password)) {
                    log("mauvais mot de passe pour " + nom);
                    envoyerErreur(resp, Strings.get("acces_refuse"));
                    return;
                }
            } catch (NoSuchAlgorithmException ex) {
                log("NoSuchAlgorithmException: " + ex.getMessage());
                envoyerErreur(resp, Strings.get("acces_refuse"));
                return;
            }
            
            if (!administrateurAutorise(user, resp))
                return;
            
            session.setAttribute("user", user);
            log(Strings.get("administration_login") + " " + user.username);
            envoyerPageEntree(resp);
            return;
        } else if (req.getParameter("action") != null) {
            final HttpSession session = req.getSession(false);
            if (session == null) {
                log(Strings.get("erreur_pas_de_session") + " " + req.getRequestURL());
                envoyerErreur(resp, Strings.get("session_expiree"));
                return;
            }
            final User admin = (User)session.getAttribute("user");
            if (admin == null) {
                log("Administration: " + Strings.get("erreur_user_null") + " " + req.getRequestURL());
                envoyerErreur(resp, Strings.get("erreur_donnees_session"));
                return;
            }
            if (!administrateurAutorise(admin, resp))
                return;
            String action = req.getParameter("action");
            if ("modifier_util".equals(action)) {
                modifierUtilisateur(req, resp);
                return;
            } else if ("modifier_coord".equals(action)) {
                modifierCoordonnees(req, resp);
                return;
            } else if ("fermer".equals(action)) {
                fermerCompte(req, resp);
                return;
            } else if ("supprimer_util".equals(action)) {
                supprimerUtilisateur(req, resp);
                return;
            } else if ("generer_site".equals(action)) {
                genererSite(req, resp, false);
                return;
            } else if ("generer_site_public".equals(action)) {
                genererSite(req, resp, true);
                return;
            } else if ("deconnexion".equals(action)) {
                deconnexion(session, resp);
                return;
            } else {
                envoyerErreur(resp, Strings.get("requete_non_comprise"));
                return;
            }
        }
        envoyerErreur(resp, Strings.get("requete_non_comprise"));
    }
    
    /**
     * Paramètres possibles pour une requête GET (un seul paramètre est nécessaire) :
     * numutilisateur : numéro de l'utilisateur
     * page : menu|inscriptions|utilisateurs|generation
     * comptes : ouverts|tous (ouverts par défaut)
     * tri : champ à utiliser pour trier la liste (par défaut: nom)
     * numcoord : numéro de coordonnées à éditer
     */
    public void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        final HttpSession session = req.getSession(false);
        if (session == null) {
            if (req.getQueryString() == null) {
                envoyerPageConnexion(req, resp);
                return;
            }
            log(Strings.get("erreur_pas_de_session") + " " + req.getRequestURL());
            envoyerErreur(resp, Strings.get("session_expiree"));
            return;
        }
        final User admin = (User)session.getAttribute("user");
        if (admin == null) {
            if (req.getQueryString() == null) {
                envoyerPageConnexion(req, resp);
                return;
            }
            log("Administration: " + Strings.get("erreur_user_null") + " " + req.getRequestURL());
            envoyerErreur(resp, Strings.get("erreur_donnees_session"));
            return;
        }
        if (!administrateurAutorise(admin, resp))
            return;
        final String numutilisateur = req.getParameter("numutilisateur");
        final String page = req.getParameter("page");
        final String numcoord = req.getParameter("numcoord");
        final String liste = req.getParameter("liste");
        
        if (numutilisateur != null) {
            final User utilisateur = chercherUser(numutilisateur);
            if (utilisateur == null) {
                log("Administration.doGet: numutilisateur: " + Strings.get("erreur_no_utilisateur") + ": " + numutilisateur);
                envoyerErreur(resp, Strings.get("erreur_no_utilisateur"));
                return;
            }
            envoyerPageUtilisateur(resp, utilisateur, liste);
        } else if (page != null) {
            if ("menu".equals(page))
                envoyerPageEntree(resp);
            else if ("inscriptions".equals(page))
                envoyerListeInscriptions(resp);
            else if ("utilisateurs".equals(page))
                envoyerListeUtilisateurs(resp, req.getParameter("comptes"), req.getParameter("tri"));
            else if ("generation".equals(page))
                envoyerPageGenerationSite(resp);
            else
                envoyerErreur(resp, "Erreur: page inconnue");
            return;
        } else if (numcoord != null) {
            final Coord coord = chercherCoord(numcoord);
            if (coord == null) {
                log("Administration.doGet: numcoord: " + Strings.get("erreur_no_coord") + ": " + numcoord);
                envoyerErreur(resp, Strings.get("erreur_no_coord"));
                return;
            }
            envoyerPageCoordonnees(resp, coord);
        } else
            envoyerPageEntree(resp);
    }
    
    public User chercherUser(final String numuser) {
        final User finduser = new User();
        finduser.numero = numuser;
        try {
            Vector<User> v = userDB.findUsers(finduser);
            if (v.size() != 0) {
                User tuteur = v.get(0);
                return(tuteur);
            }
        } catch (JDBException ex) {
            log("Administration: chercherUser: JDBException: " + ex.getMessage());
        }
        log("Administration: chercherUser: pas d'utilisateur " + numuser);
        return(null);
    }
    
    public Coord chercherCoord(final String numcoord) {
        final Coord findcoord = new Coord();
        findcoord.numero = numcoord;
        try {
            Vector<Coord> v = coordDB.findCoords(findcoord);
            if (v.size() != 0) {
                Coord res = v.get(0);
                return(res);
            }
        } catch (JDBException ex) {
            log("Administration: chercherCoord: JDBException: " + ex.getMessage());
        }
        log("Administration: chercherCoord: pas de coordonnées " + numcoord);
        return(null);
    }
    
    public void enteteHTML(final PrintWriter pw, final String titre) {
        pw.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">");
        pw.println("<html>");
        pw.println("<head>");
        pw.println(" <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">");
        pw.println(" <title>" + titre + "</title>");
        pw.println(" <link href=\"paellos.css\" rel=\"stylesheet\" type=\"text/css\">");
        pw.println("</head>");
        pw.println("<body>");
        pw.println("<div class=\"contenu_admin\">");
        pw.println("<img src=\"logo_paellos.png\" alt=\"logo\" align=\"right\">");
    }
    
    public void piedHTML(final PrintWriter pw) {
        pw.println("</div>");
        pw.println("</body>");
        pw.println("</html>");
        pw.flush();
    }
    
    public void envoyerErreur(final HttpServletResponse resp, final String errmsg) {
        try {
            resp.setHeader("Pragma", "no-cache");
            resp.setHeader("Cache-Control", "no-cache");
            resp.setHeader("Expires", "0");
            resp.setContentType("text/html; charset=UTF-8");
            
            final PrintWriter pw = resp.getWriter();
            enteteHTML(pw, Strings.get("titre_erreur"));
            menu(pw, null, null);
            pw.println("<p>" + Strings.get("une_erreur") + " " + errmsg + "</p>");
            pw.println("<p>" + Strings.get("bouton_retour") + "</p>");
            piedHTML(pw);
        } catch (IOException ex) {
            log("Administration: envoyerErreur: " + ex.getMessage());
        }
    }
    
    public boolean administrateurAutorise(final User u, final HttpServletResponse resp) {
        // obtention date d'aujourd'hui
        final Date now = new Date();
        final SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm");
        final String date = formatter.format(now);
        
        // test fermeture de compte
        if (u.datefermeture != null && !"".equals(u.datefermeture)) {
            Date fermeture;
            try {
                fermeture = formatter.parse(u.datefermeture);
                if (now.after(fermeture)) {
                    log("Administration: " + Strings.get("utilisation_compte_ferme") + " " + u.username);
                    envoyerErreur(resp, Strings.get("acces_refuse_compte_ferme"));
                    return(false);
                }
            } catch (ParseException ex) {
                log("Administration: datefermeture ParseException: " + ex.getMessage());
            }
        }
        
        // test fonction autorisée: administrateur ou système
        if (!"administrateur".equals(u.fonction) && !"système".equals(u.fonction)) {
            log(Strings.get("admin_interdite_pour") + " " + u.username);
            envoyerErreur(resp, Strings.get("acces_refuse"));
            return(false);
        }
        return(true);
    }
    
    public void envoyerPageConnexion(final HttpServletRequest req, final HttpServletResponse resp) {
        try {
            getServletContext().getRequestDispatcher("/WEB-INF/jsp/administration.jsp").forward(req, resp);
        } catch (ServletException ex) {
            log("Administration: envoyerPageConnexion: " + ex.getClass() + ": " + ex.getMessage());
            envoyerErreur(resp, Strings.get("erreur_cf_log"));
        } catch (IOException ex) {
            log("Administration: envoyerPageConnexion: " + ex.getClass() + ": " + ex.getMessage());
            envoyerErreur(resp, Strings.get("erreur_cf_log"));
        }
    }
    
    public void menu(final PrintWriter pw, final String pageActuelle, final String tri) {
        final String opttri;
        if (tri != null)
            opttri = "&amp;tri=" + tri;
        else
            opttri = "";
        if (!"menu".equals(pageActuelle))
            pw.println("<div class=\"menu\">");
        pw.println("<ul>");
        pw.print("<li><a href=\"Administration?page=inscriptions\"");
        if ("inscriptions".equals(pageActuelle))
            pw.print(" class=\"selected\"");
        pw.println(">" + Strings.get("nouvelles_inscriptions") + "</a></li>");
        pw.print("<li><a href=\"Administration?page=utilisateurs" + opttri + "\"");
        if ("utilisateurs".equals(pageActuelle))
            pw.print(" class=\"selected\"");
        pw.println(">" + Strings.get("util_valides_cpt_ouverts") + "</a></li>");
        pw.print("<li><a href=\"Administration?page=utilisateurs&amp;comptes=tous" + opttri + "\"");
        if ("tous".equals(pageActuelle))
            pw.print(" class=\"selected\"");
        pw.println(">" + Strings.get("tous_util_valides") + "</a></li>");
        pw.print("<li><a href=\"Administration?page=generation\"");
        if ("generation".equals(pageActuelle))
            pw.print(" class=\"selected\"");
        pw.println(">" + Strings.get("generation_site") + "</a></li>");
        pw.println("<li><a href=\"" + URLServeur + pageLogin + "\">" + Strings.get("acces_site") + "</a></li>");
        pw.println("<li>");
        pw.println("<form method=\"post\" action=\"Administration\">");
        pw.println("<input type=\"hidden\" name=\"action\" value=\"deconnexion\">");
        pw.println("<input type=\"submit\" value=\"" + Strings.get("deconnexion") + "\">");
        pw.println("</form>");
        pw.println("</li>");
        pw.println("</ul>");
        if (!"menu".equals(pageActuelle))
            pw.println("</div>");
    }
    
    public void envoyerPageEntree(final HttpServletResponse resp) {
        /*
        try {
            resp.setContentType("text/html; charset=UTF-8");
            final Writer out = resp.getWriter();
            final PrintWriter pw = new PrintWriter(out);
            enteteHTML(pw, Strings.get("titre_administration"));
            pw.println("<h1>" + Strings.get("titre_administration") + "</h1>");
            menu(pw, "menu", null);
            piedHTML(pw);
        } catch (IOException ex) {
            log("Administration: envoyerPageEntree: " + ex.getMessage());
        }
        */
        envoyerListeInscriptions(resp);
    }
    
    public void envoyerListeUtilisateurs(final HttpServletResponse resp, final String comptes, final String tri) {
        try {
            resp.setHeader("Pragma", "no-cache");
            resp.setHeader("Cache-Control", "no-cache");
            resp.setHeader("Expires", "0");
            resp.setContentType("text/html; charset=UTF-8");
            final Writer out = resp.getWriter();
            final PrintWriter pw = new PrintWriter(out);
            enteteHTML(pw, Strings.get("admin_liste_util"));
            final String liste;
            if ("tous".equals(comptes))
                liste = "tous";
            else
                liste = "utilisateurs";
            menu(pw, liste, tri);
            pw.println("<h1>" + Strings.get("titre_administration") + "</h1>");
            pw.println("<h2>" + Strings.get("titre_utilisateurs") + "</h2>");
            pw.println("<table border=\"1\" cellpadding=\"2\" cellspacing=\"0\">");
            String debut_lien = "Administration?page=utilisateurs";
            if ("tous".equals(comptes))
                debut_lien += "&amp;comptes=tous";
            pw.print(" <tr><th><a href=\"" + debut_lien + "&amp;tri=username\">" + Strings.get("nom_util") + "</a></th>");
            pw.print("<th><a href=\"" + debut_lien + "&amp;tri=nom\">" + Strings.get("nom_famille") + "</a></th>");
            pw.print("<th><a href=\"" + debut_lien + "&amp;tri=prenom\">" + Strings.get("prenom") + "</a></th>");
            pw.print("<th><a href=\"" + debut_lien + "&amp;tri=email\">" + Strings.get("email") + "</a></th>");
            pw.print("<th><a href=\"" + debut_lien + "&amp;tri=fonction\">" + Strings.get("fonction") + "</a></th>");
            if (noms_formations != null && noms_formations.length > 1)
                pw.println("<th><a href=\"" + debut_lien + "&amp;tri=formation\">" + Strings.get("formation") + "</a></th>");
            pw.println("<th>" + Strings.get("actions") + "</th></tr>");
            final User finduser = new User();
            //finduser.fonction = "étudiant";
            finduser.valide = "1";
            if (noms_formations != null && noms_formations.length == 1)
                finduser.formation = noms_formations[0];
            final SimpleDateFormat isoformatter = new SimpleDateFormat("yyyy-MM-dd");
            final String maintenant = isoformatter.format(new Date());
            try {
                final Vector<User> v = userDB.findUsers(finduser);
                final Vector<User> selection = new Vector<User>();
                for (User utilisateur : v) {
                    boolean formationok;
                    if (noms_formations == null || utilisateur.formation == null)
                        formationok = true;
                    else
                        formationok = Arrays.asList(noms_formations).contains(utilisateur.formation);
                    boolean dateok = (utilisateur.datefermeture == null || "".equals(utilisateur.datefermeture) ||
                        "tous".equals(comptes) || utilisateur.datefermeture.compareTo(maintenant) > 0);
                    if (formationok && dateok)
                        selection.add(utilisateur);
                }
                trier(selection, tri);
                boolean t1 = true;
                for (User utilisateur : selection) {
                    pw.println(" <tr>");
                    pw.println("  <td class=\"" + (t1?"t1":"t2") + "\"><a href=\"Administration?numutilisateur="+utilisateur.numero+"&amp;liste=" + liste + "\">" + utilisateur.username + "</a></td>");
                    pw.println("  <td class=\"" + (t1?"t1":"t2") + "\">" + utilisateur.nom + "</td>");
                    pw.println("  <td class=\"" + (t1?"t1":"t2") + "\">" + utilisateur.prenom + "</td>");
                    pw.println("  <td class=\"petit " + (t1?"t1":"t2") + "\">" + utilisateur.email + "</td>");
                    pw.println("  <td class=\"" + (t1?"t1":"t2") + "\">" + utilisateur.fonction + "</td>");
                    if (noms_formations != null && noms_formations.length > 1)
                        pw.println("  <td class=\"" + (t1?"t1":"t2") + "\">" + utilisateur.formation + "</td>");
                    pw.println("  <td class=\"" + (t1?"t1":"t2") + "\">");
                    if (!"tous".equals(comptes) || utilisateur.datefermeture == null || "".equals(utilisateur.datefermeture)) {
                        pw.println("    <form method=\"post\" action=\"Administration\" class=\"inline\">");
                        pw.println("      <input type=\"hidden\" name=\"action\" value=\"fermer\">");
                        pw.println("      <input type=\"hidden\" name=\"numutilisateur\" value=\"" + utilisateur.numero + "\">");
                        if (comptes != null)
                            pw.println("      <input type=\"hidden\" name=\"comptes\" value=\"" + comptes + "\">");
                        if (tri != null)
                            pw.println("      <input type=\"hidden\" name=\"tri\" value=\"" + tri + "\">");
                        pw.println("      <input type=\"submit\" value=\"" + Strings.get("fermer_compte") + "\">");
                        pw.println("    </form>");
                    }
                    pw.println("    <form method=\"post\" action=\"Administration\" class=\"inline\">");
                    pw.println("      <input type=\"hidden\" name=\"action\" value=\"supprimer_util\">");
                    pw.println("      <input type=\"hidden\" name=\"numutilisateur\" value=\"" + utilisateur.numero + "\">");
                    pw.println("      <input type=\"hidden\" name=\"page\" value=\"utilisateurs\">");
                    if (comptes != null)
                        pw.println("      <input type=\"hidden\" name=\"comptes\" value=\"" + comptes + "\">");
                    if (tri != null)
                        pw.println("      <input type=\"hidden\" name=\"tri\" value=\"" + tri + "\">");
                    pw.println("      <input type=\"submit\" value=\"" + Strings.get("supprimer") + "\">");
                    pw.println("    </form>");
                    pw.println("  </td>");
                    pw.println(" </tr>");
                    t1 = !t1;
                }
            } catch (JDBException ex) {
                log("JDBException: " + ex.getMessage());
            }
            pw.println("</table>");
            piedHTML(pw);
        } catch (IOException ex) {
            log("Administration: envoyerListeUtilisateurs: " + ex.getMessage());
        }
    }
    
    public void trier(final Vector<User> liste, final String tri) {
        if (tri == null || "nom".equals(tri)) {
            Collections.sort(liste);
            return;
        }
        Collections.sort(liste, new UserComparator(tri));
    }
    
    class UserComparator implements Comparator<User> {
        String tri;
        public UserComparator(final String tri) {
            this.tri = tri;
        }
        public int compare(User u1, User u2) {
            int n_champ = 2; // nom
            if (tri != null) {
                String[] fields = UserDB.fields;
                for (int i=0; i<fields.length; i++)
                    if (tri.equals(fields[i]))
                        n_champ = i;
            }
            final Object[] champs_u1 = u1.getFieldValues();
            String c1 = (String)champs_u1[n_champ];
            final Object[] champs_u2 = u2.getFieldValues();
            String c2 = (String)champs_u2[n_champ];
            if (c1 == null) {
                if (c2 == null)
                    return(0);
                else
                    return(-1);
            }
            if (c2 == null)
                return(1);
            int comp = c1.compareToIgnoreCase(c2);
            if (n_champ == 2 || comp != 0)
                return(comp);
            c1 = u1.nom;
            c2 = u2.nom;
            if (c1 == null) {
                if (c2 == null)
                    return(0);
                else
                    return(-1);
            }
            if (c2 == null)
                return(1);
            return(c1.compareToIgnoreCase(c2));
        }
    }
    
    public void envoyerListeInscriptions(final HttpServletResponse resp) {
        try {
            resp.setHeader("Pragma", "no-cache");
            resp.setHeader("Cache-Control", "no-cache");
            resp.setHeader("Expires", "0");
            resp.setContentType("text/html; charset=UTF-8");
            
            final Writer out = resp.getWriter();
            final PrintWriter pw = new PrintWriter(out);
            enteteHTML(pw, Strings.get("admin_inscriptions"));
            menu(pw, "inscriptions", null);
            pw.println("<h1>" + Strings.get("titre_administration") + "</h1>");
            pw.println("<h2>" + Strings.get("titre_inscriptions") + "</h2>");
            // à faire: bouton supprimer avec confirmation
            final User finduser = new User();
            //finduser.fonction = "étudiant";
            finduser.valide = "0";
            if (noms_formations != null && noms_formations.length == 1)
                finduser.formation = noms_formations[0];
            final Vector<User> selection = new Vector<User>();
            try {
                final Vector<User> v = userDB.findUsers(finduser);
                for (User utilisateur : v) {
                    boolean formationok;
                    if (noms_formations == null || utilisateur.formation == null)
                        formationok = true;
                    else
                        formationok = Arrays.asList(noms_formations).contains(utilisateur.formation);
                    if (formationok)
                        selection.add(utilisateur);
                }
            } catch (JDBException ex) {
                log("JDBException: " + ex.getMessage());
            }
            if (selection.size() == 0)
                pw.println("<p>" + Strings.get("aucune") + "</p>");
            else {
                Collections.sort(selection);
                pw.println("<table border=\"1\" cellpadding=\"2\" cellspacing=\"0\">");
                pw.print(" <tr><th>" + Strings.get("nom_util") + "</th><th>" + Strings.get("nom_famille") +
                    "</th><th>" + Strings.get("prenom") + "</th><th>" + Strings.get("email") + "</th><th>" +
                    Strings.get("fonction") + "</th>");
                if (noms_formations != null && noms_formations.length > 1)
                    pw.print("<th>" + Strings.get("formation") + "</th>");
                pw.println("<th>" + Strings.get("actions") + "</th></tr>");
                for (User utilisateur : selection) {
                    pw.println(" <tr>");
                    pw.println("  <td><a href=\"Administration?numutilisateur="+utilisateur.numero+"&amp;liste=inscriptions\">" + utilisateur.username + "</a></td>");
                    pw.println("  <td>" + utilisateur.nom + "</td>");
                    pw.println("  <td>" + utilisateur.prenom + "</td>");
                    pw.println("  <td class=\"petit\">" + utilisateur.email + "</td>");
                    pw.println("  <td>" + utilisateur.fonction + "</td>");
                    if (noms_formations != null && noms_formations.length > 1)
                        pw.println("  <td>" + utilisateur.formation + "</td>");
                    pw.println("  <td>");
                    pw.println("    <form method=\"post\" action=\"Administration\" class=\"inline\">");
                    pw.println("      <input type=\"hidden\" name=\"action\" value=\"supprimer_util\">");
                    pw.println("      <input type=\"hidden\" name=\"numutilisateur\" value=\"" + utilisateur.numero + "\">");
                    pw.println("      <input type=\"hidden\" name=\"page\" value=\"inscriptions\">");
                    pw.println("      <input type=\"submit\" value=\"" + Strings.get("supprimer") + "\">");
                    pw.println("    </form>");
                    pw.println("  </td>");
                    pw.println(" </tr>");
                }
                pw.println("</table>");
            }
            piedHTML(pw);
        } catch (IOException ex) {
            log("envoyerListeUtilisateurs: " + ex.getMessage());
        }
    }
    
    public void envoyerPageUtilisateur(final HttpServletResponse resp, final User utilisateur, final String liste) {
        try {
            resp.setHeader("Pragma", "no-cache");
            resp.setHeader("Cache-Control", "no-cache");
            resp.setHeader("Expires", "0");
            resp.setContentType("text/html; charset=UTF-8");
            final Writer out = resp.getWriter();
            final PrintWriter pw = new PrintWriter(out);
            enteteHTML(pw, Strings.get("admin_edition_util"));
            menu(pw, liste, null);
            pw.println("<h1>" + Strings.get("titre_administration") + "</h1>");
            pw.println("<h2>" + Strings.get("titre_utilisateur") + "</h2>");
            pw.println("<form method=\"post\" action=\"Administration\">");
            pw.println("<input type=\"hidden\" name=\"action\" value=\"modifier_util\">");
            pw.println("<input type=\"hidden\" name=\"numero\" value=\"" + utilisateur.numero + "\">");
            if (liste != null)
                pw.println("<input type=\"hidden\" name=\"liste\" value=\"" + liste +"\">");
            pw.println("<table border=\"1\" cellpadding=\"2\" cellspacing=\"0\">");
            pw.println("<tr><td>" + Strings.get("nom_util") + "</td>");
            pw.println("<td><input type=\"text\" name=\"username\" size=\"15\" maxlength=\"15\" value=\"" +
                entites(utilisateur.username) + "\"></td></tr>");
            pw.println("<tr><td>" + Strings.get("nom_famille") + "</td>");
            pw.println("<td><input type=\"text\" name=\"nom\" size=\"30\" maxlength=\"30\" value=\"" +
                entites(utilisateur.nom) + "\"></td></tr>");
            pw.println("<tr><td>" + Strings.get("prenom") + "</td>");
            pw.println("<td><input type=\"text\" name=\"prenom\" size=\"20\" maxlength=\"20\" value=\"" +
                entites(utilisateur.prenom) + "\"></td></tr>");
            pw.println("<tr><td>" + Strings.get("mdp") + "</td>");
            pw.println("<td><input type=\"password\" name=\"password\" size=\"20\" maxlength=\"20\" value=\"\"></td></tr>");
            pw.println("<tr><td>" + Strings.get("email") + "</td>");
            pw.println("<td><input type=\"text\" name=\"email\" size=\"50\" maxlength=\"50\" value=\"" +
                entites(utilisateur.email) + "\"></td></tr>");
            pw.println("<tr><td>" + Strings.get("fonction") + "</td>");
            pw.println("<td><select name=\"fonction\">");
            for (int i=0; i<fonctionval.length; i++) {
                final String fonction = fonctionval[i];
                pw.print("  <option value=\"" + fonction + "\"");
                if (fonction.equals(utilisateur.fonction))
                    pw.print(" selected");
                pw.println(">" + Strings.get(fonctionstr[i]) + "</option>");
            }
            pw.println("</select></td></tr>");
            pw.println("<tr><td>" + Strings.get("naissance") + "</td>");
            pw.println("<td><input type=\"text\" name=\"naissance\" size=\"10\" maxlength=\"10\" value=\"" +
                entites(utilisateur.naissance) + "\"></td></tr>");
            pw.println("<tr><td>" + Strings.get("formation") + "</td>");
            pw.println("<td><input type=\"text\" name=\"formation\" size=\"15\" maxlength=\"15\" value=\"" +
                entites(utilisateur.formation) + "\"></td></tr>");
            pw.println("<tr><td>" + Strings.get("niveau") + "</td>");
            pw.println("<td><select name=\"niveau\">");
            for (String niveau : niveauval) {
                pw.print("  <option value=\"" + niveau + "\"");
                if (niveau.equals(utilisateur.niveau) || ("".equals(niveau) && utilisateur.niveau == null))
                    pw.print(" selected");
                pw.println(">" + niveau + "</option>");
            }
            pw.println("</select></td></tr>");
            pw.println("<tr><td>" + Strings.get("modules") + "</td>");
            pw.println("<td><input type=\"text\" name=\"modules\" size=\"20\" maxlength=\"20\" value=\"" +
                entites(utilisateur.modules) + "\"></td></tr>");
            pw.println("<tr><td>" + Strings.get("matiere") + "</td>");
            pw.println("<td><input type=\"text\" name=\"matiere\" size=\"40\" maxlength=\"40\" value=\"" +
                entites(utilisateur.matiere) + "\"></td></tr>");
            pw.println("<tr><td>" + Strings.get("classe") + "</td>");
            pw.println("<td><input type=\"text\" name=\"classe\" size=\"40\" maxlength=\"40\" value=\"" +
                entites(utilisateur.classe) + "\"></td></tr>");
            pw.println("<tr><td>" + Strings.get("date_inscription") + "</td>");
            pw.println("<td><input type=\"text\" name=\"dateinscription\" size=\"10\" maxlength=\"10\" value=\"" +
                entites(utilisateur.dateinscription) + "\"></td></tr>");
            pw.println("<tr><td>" + Strings.get("date_fermeture") + "</td>");
            pw.println("<td><input type=\"text\" name=\"datefermeture\" size=\"10\" maxlength=\"10\" value=\"" +
                entites(utilisateur.datefermeture) + "\"></td></tr>");
            User tuteur;
            if (utilisateur.numtuteur != null && !"".equals(utilisateur.numtuteur) && !"0".equals(utilisateur.numtuteur))
                tuteur = chercherUser(utilisateur.numtuteur);
            else
                tuteur = null;
            final User findtuteur = new User();
            findtuteur.fonction = "tuteur";
            findtuteur.valide = "1";
            try {
                // recherche des tuteurs valides
                final Vector<User> tuteursValides = userDB.findUsers(findtuteur);
                // sélection des tuteurs dont le compte n'est pas fermé
                final Vector<User> tuteurs = new Vector<User>();
                final SimpleDateFormat isoformatter = new SimpleDateFormat("yyyy-MM-dd");
                final String maintenant = isoformatter.format(new Date());
                for (User t : tuteursValides) {
                    if (t.datefermeture == null || "".equals(t.datefermeture) ||
                            t.datefermeture.compareTo(maintenant) > 0)
                        tuteurs.add(t);
                }
                // ajout des supertuteurs
                findtuteur.fonction = "supertuteur";
                final Vector<User> supertuteurs = userDB.findUsers(findtuteur);
                tuteurs.addAll(supertuteurs);
                // affichage
                pw.print("<tr><td>" + Strings.get("titre_tuteur") + "</td><td>");
                if (tuteurs.size() != 0) {
                    pw.println("<select name=\"numtuteur\">");
                    pw.println("<option value=\"\"");
                    if (tuteur == null)
                        pw.print(" selected");
                    pw.println(">" + Strings.get("aucun") + "</option>");
                    Collections.sort(tuteurs);
                    for (User tuteuri : tuteurs) {
                        pw.print("<option value=\"" + tuteuri.numero + "\"");
                        if (tuteur != null && tuteuri.numero.equals(tuteur.numero))
                            pw.print(" selected");
                        pw.println(">" + entites(tuteuri.prenom + " " + tuteuri.nom) + "</option>");
                    }
                    pw.println("</select>");
                } else
                    pw.print(Strings.get("aucun"));
                pw.println("</td></tr>");
            } catch (JDBException ex) {
                log("Administration: envoyerPageUtilisateur: findUsers: JDBException: " + ex.getMessage());
                if (tuteur == null)
                    pw.println("<tr><td>" + Strings.get("titre_tuteur") + "</td><td>*" + Strings.get("erreur_majuscules") + "*</td></tr>");
                else
                    pw.println("<tr><td>" + Strings.get("titre_tuteur") + "</td><td>*" + Strings.get("erreur_majuscules") + "* " +
                        entites(tuteur.prenom + " " + tuteur.nom) + "</td></tr>");
            }
            pw.println("<tr><td>" + Strings.get("centre") + "</td>");
            pw.println("<td><input type=\"text\" name=\"centre\" size=\"20\" maxlength=\"20\" value=\"" +
                entites(utilisateur.centre) + "\"></td></tr>");
            pw.println("<tr><td>Valide</td><td><input name=\"valide\" type=\"checkbox\"" + ("1".equals(utilisateur.valide) ? " checked" : "") + "></td>");
            pw.println("</table>");
            if ("0".equals(utilisateur.valide) && "inscriptions".equals(liste)) {
                // vérif s'il existe déjà un utilisateur validé avec le même nom et prénom
                final User usertest = new User();
                usertest.nom = utilisateur.nom;
                usertest.prenom = utilisateur.prenom;
                usertest.valide = "1";
                try {
                    final Vector<User> fusers = userDB.findUsers(usertest);
                    if (fusers.size() > 0) {
                        final User existant = fusers.get(0);
                        pw.print("<p><em>" + Strings.get("existe_meme_nom") + "</em>");
                        if (existant.formation != null && !"".equals(existant.formation) &&
                                !existant.formation.equals(utilisateur.formation))
                            pw.print(" (" + existant.formation + ")");
                        pw.print(" <a href=\"Administration?numutilisateur=" + existant.numero + "\">");
                        pw.println(existant.username + "</a></p>");
                    }
                } catch (JDBException ex) {
                }
            }
            pw.println("<p><input type=\"submit\" value=\"" + Strings.get("enregistrer_champs") + "\"> ");
            if (liste != null) {
                pw.print(" <a href=\"Administration?page=");
                if ("inscriptions".equals(liste) || "utilisateurs".equals(liste))
                    pw.print(liste);
                else if ("tous".equals(liste))
                    pw.print("utilisateurs&amp;comptes=tous");
                pw.print("\">" + Strings.get("annuler") + "</a>");
            }
            pw.println("</p>");
            pw.println("</form>");
            
            pw.println("<hr>");
            pw.println("<h2>" + Strings.get("titre_coordonnees") + "</h2>");
            final Coord findcoord = new Coord(utilisateur.numero);
            try {
                final Vector<Coord> coordonnees = coordDB.findCoords(findcoord);
                if (coordonnees.size() != 0) {
                    for (Coord coord : coordonnees) {
                        pw.println("<table border=\"1\" cellpadding=\"2\" cellspacing=\"0\">");
                        pw.print("<tr><td rowspan=\"2\">" + Strings.get("rue") + "</td><td>" + entites(coord.rue1) + "</td></tr>");
                        pw.print("<tr><td>" + entites(coord.rue2) + "</td></tr>");
                        pw.print("<tr><td>" + Strings.get("code_postal") + "</td><td>" + entites(coord.codepostal) + "</td></tr>");
                        pw.print("<tr><td>" + Strings.get("ville") + "</td><td>" + entites(coord.ville) + "</td></tr>");
                        pw.print("<tr><td>" + Strings.get("pays") + "</td><td>" + entites(coord.pays) + "</td></tr>");
                        pw.print("<tr><td>" + Strings.get("telephone") + "</td><td>" + entites(coord.telephone) + "</td></tr>");
                        pw.print("<tr><td>" + Strings.get("connexion") + "</td><td>" + entites(coord.connection) + "</td></tr>");
                        pw.print("<tr><td>" + Strings.get("acces") + "</td><td>" + entites(coord.acces) + "</td></tr>");
                        pw.println("</table>");
                        pw.println("<p><a href=\"Administration?numcoord=" + coord.numero + "\">" + Strings.get("editer_coord") + "</a></p>");
                    }
                }
                // à faire: ajouter/supprimer des coordonnées
            } catch (JDBException ex) {
                log("Administration: envoyerPageUtilisateur: findCoords: JDBException: " + ex.getMessage());
                pw.println("<p>" + Strings.get("erreur_cf_log") + "</p>");
            }
            
            /* via la case Valide
            if ("0".equals(utilisateur.valide)) {
                pw.println("<form method=\"post\" action=\"Administration\">");
                pw.println("  <input type=\"hidden\" name=\"numutilisateur\" value=\"" + utilisateur.numero + "\">");
                pw.println("  <input type=\"hidden\" name=\"action\" value=\"valider\">");
                pw.println("  <input type=\"submit\" value=\"Valider l'inscription\"></p>");
                pw.println("</form>");
            }
            */
            
            if ("étudiant".equals(utilisateur.fonction)) {
                pw.println("<hr>");
                pw.println("<p><a href=\"Lectures?numetudiant=" + utilisateur.numero + "\">" + Strings.get("lectures_cet_etudiant") + "</a></p>");
            }
            piedHTML(pw);
        } catch (IOException ex) {
            log("envoyerListeUtilisateurs: " + ex.getMessage());
        }
    }
    
    public void modifierUtilisateur(final HttpServletRequest req, final HttpServletResponse resp) {
        String numero = req.getParameter("numero");
        String username = req.getParameter("username");
        String nom = req.getParameter("nom");
        String prenom = req.getParameter("prenom");
        String password = req.getParameter("password");
        String email = req.getParameter("email");
        String fonction = req.getParameter("fonction");
        String naissance = req.getParameter("naissance");
        String formation = req.getParameter("formation");
        String niveau = req.getParameter("niveau");
        String modules = req.getParameter("modules");
        String matiere = req.getParameter("matiere");
        String classe = req.getParameter("classe");
        String dateinscription = req.getParameter("dateinscription");
        String datefermeture = req.getParameter("datefermeture");
        String numtuteur = req.getParameter("numtuteur");
        String centre = req.getParameter("centre");
        String valide = req.getParameter("valide");
        String liste = req.getParameter("liste");
        final User utilisateur = chercherUser(numero);
        if (utilisateur == null) {
            envoyerErreur(resp, Strings.get("utilisateur_introuvable"));
            return;
        }
        // vérification des valeurs
        if (username == null) {
            envoyerErreur(resp, Strings.get("manque_nom_utilisateur"));
            return;
        }
        if (utilisateur.username != null && !utilisateur.username.equals(username)) {
            if (username.length() > 15) {
                envoyerErreur(resp, Strings.get("nom_util_trop_long"));
                return;
            }
            if (username.indexOf(' ') != -1) {
                envoyerErreur(resp, Strings.get("espaces_nom_utilisateur"));
                return;
            }
            final String autorises = "abcdefghijklmnopqrstuvwxyz0123456789.-_";
            for (int i=0; i<username.length(); i++)
                if (autorises.indexOf(username.charAt(i)) == -1) {
                    envoyerErreur(resp, Strings.get("caracteres_nom_utilisateurs"));
                    return;
                }
            final User usertest = new User(username);
            try {
                final Vector<User> fusers = userDB.findUsers(usertest);
                if (fusers.size() > 0) {
                    envoyerErreur(resp, Strings.get("nom_deja_pris"));
                    return;
                }
            } catch (JDBException ex) {
                envoyerErreur(resp, Strings.get("erreur_verif_nom_utilisateur") + ": " + ex.getMessage());
                return;
            }
        }
        if (nom != null && nom.length() > 30) {
            envoyerErreur(resp, Strings.get("nom_famille_trop_long"));
            return;
        }
        if (prenom != null && prenom.length() > 20) {
            envoyerErreur(resp, Strings.get("prenom_trop_long"));
            return;
        }
        if (password != null && !"".equals(password)) {
            if (password.length() < 5) {
                envoyerErreur(resp, Strings.get("mdp_trop_court"));
                return;
            }
            if (password.length() > 20) {
                envoyerErreur(resp, Strings.get("mdp_trop_long"));
                return;
            }
            try {
                password = userDB.encryptSHA256(password);
            } catch (NoSuchAlgorithmException ex) {
                log("modifierUtilisateur: NoSuchAlgorithmException: " + ex.getMessage());
                password = userDB.encrypt(password);
            }
        }
        if (email != null && email.length() > 50) {
            envoyerErreur(resp, Strings.get("mail_trop_long"));
            return;
        }
        final boolean trouve = Arrays.asList(fonctionval).contains(fonction);
        if (!trouve) {
            envoyerErreur(resp, Strings.get("fonction_inconnue") + " " + fonction);
            return;
        }
        if (naissance != null && naissance.length() > 0 && naissance.length() != 10) {
            envoyerErreur(resp, Strings.get("format_date_naissance"));
            return;
        }
        if (formation != null && formation.length() > 15) {
            envoyerErreur(resp, Strings.get("nom_formation_trop_long"));
            return;
        }
        if (niveau != null && niveau.length() > 20) {
            envoyerErreur(resp, Strings.get("niveau_trop_long"));
            return;
        }
        if (modules != null && modules.length() > 20) {
            envoyerErreur(resp, Strings.get("nom_modules_trop_long"));
            return;
        }
        if (matiere != null && matiere.length() > 40) {
            envoyerErreur(resp, Strings.get("nom_matiere_trop_long"));
            return;
        }
        if (classe != null && classe.length() > 40) {
            envoyerErreur(resp, Strings.get("nom_classe_trop_long"));
            return;
        }
        if (dateinscription != null && dateinscription.length() > 0 && dateinscription.length() != 10) {
            envoyerErreur(resp, Strings.get("format_date_inscription"));
            return;
        }
        if (datefermeture != null && datefermeture.length() > 0 && datefermeture.length() != 10) {
            envoyerErreur(resp, Strings.get("format_date_fermeture"));
            return;
        }
        if (numtuteur != null && !"".equals(numtuteur) && !"0".equals(numtuteur)) {
            final User tuteur = chercherUser(numtuteur);
            if (tuteur == null) {
                envoyerErreur(resp, Strings.get("tuteur_introuvable") + " " + numtuteur);
                return;
            }
        }
        if (centre != null && centre.length() > 20) {
            envoyerErreur(resp, Strings.get("centre_trop_long"));
            return;
        }
        utilisateur.username = username;
        utilisateur.nom = nom;
        utilisateur.prenom = prenom;
        if (password != null && !"".equals(password))
            utilisateur.password = password;
        utilisateur.email = email;
        utilisateur.fonction = fonction;
        utilisateur.naissance = naissance;
        utilisateur.formation = formation;
        utilisateur.niveau = niveau;
        utilisateur.modules = modules;
        utilisateur.matiere = matiere;
        utilisateur.classe = classe;
        utilisateur.dateinscription = dateinscription;
        utilisateur.datefermeture = datefermeture;
        if ("".equals(numtuteur))
            utilisateur.numtuteur = "0";
        else
            utilisateur.numtuteur = numtuteur;
        utilisateur.centre = centre;
        final boolean validation = ("0".equals(utilisateur.valide) && valide != null);
        utilisateur.valide = (valide != null ? "1" : "0");
        try {
            userDB.modUser(utilisateur);
        } catch (JDBException ex) {
            log("Administration: modifierUtilisateur: JDBException: " + ex.getMessage());
            envoyerErreur(resp, Strings.get("erreur_modif_util"));
            return;
        }
        try {
            resp.setContentType("text/html; charset=UTF-8");
            final Writer out = resp.getWriter();
            final PrintWriter pw = new PrintWriter(out);
            enteteHTML(pw, Strings.get("admin_edition_util"));
            menu(pw, liste, null);
            pw.println("<h1>" + Strings.get("titre_administration") + "</h1>");
            pw.println("<h2>" + Strings.get("titre_utilisateur") + "</h2>");
            pw.println("<p>" + Strings.get("modifs_enregistrees") + "</p>");
            if (validation) {
                pw.println("<p>" + Strings.get("inscription_validee") + "</p>");
                envoyerMailConfirmation(utilisateur, req);
                pw.println("<p>" + Strings.get("mail_envoye_utilisateur") + "</p>");
                if (envoyerMailTuteur(utilisateur, req))
                    pw.println("<p>" + Strings.get("mail_envoye_tuteur") + "</p>");
            }
            if (liste != null) {
                pw.print("<p><a href=\"Administration?page=");
                if ("inscriptions".equals(liste) || "utilisateurs".equals(liste))
                    pw.print(liste);
                else if ("tous".equals(liste))
                    pw.print("utilisateurs&amp;comptes=tous");
                pw.println("\">" + Strings.get("revenir_liste") + "</a></p>");
            }
            piedHTML(pw);
        } catch (IOException ex) {
            log("Administration: modifierUtilisateur: " + ex.getMessage());
        }
    }
    
    public void envoyerMailConfirmation(final User utilisateur, final HttpServletRequest req) throws IOException {
		String subject = Strings.get("inscription_formation") + " " + titreFormation(utilisateur.formation);
        String body = Strings.get("bonjour") + "\n";
        body += "\n";
        body += Strings.get("votre_inscription_validee") + "\n";
        body += Strings.get("bienvenue_formation") + " \"" + titreFormation(utilisateur.formation) + "\".\n";
        body += "\n";
        body += Strings.get("acces_au_site") + "\n";
        body += URLServeur + pageLogin + "\n";
        body += "\n";
        body += Strings.get("votre_nom_utilisateur") + " " + utilisateur.username + "\n";
        body += Strings.get("et_votre_mdp") + "\n";
        body += Strings.get("info_changement_mdp") + "\n";
        body += "\n";
        if (!"".equals(utilisateur.numtuteur) && !"0".equals(utilisateur.numtuteur)) {
            User tuteur = chercherUser(utilisateur.numtuteur);
            if (tuteur != null)
                body += Strings.get("votre_tuteur") + " " + tuteur.prenom + " " + tuteur.nom + " (" + tuteur.email + ")\n";
        }
        final Mail courriel = new Mail(this, req.getServerName(), smtpServer, 25, emailAdmin, utilisateur.email, null);
        courriel.definirMessage(subject, body);
        courriel.envoyer(false);
    }
    
    public String titreFormation(final String nom_formation) {
        String titre_formation = nom_formation;
        if (noms_formations != null)
            for (int i=0; i<noms_formations.length; i++)
                if (noms_formations[i].equals(nom_formation)) {
                    titre_formation = titres_formations[i];
                    break;
                }
        return(titre_formation);
    }
    
    public boolean envoyerMailTuteur(final User utilisateur, final HttpServletRequest req) throws IOException {
        if (utilisateur.numtuteur == null || "".equals(utilisateur.numtuteur) || "0".equals(utilisateur.numtuteur))
            return(false);
        final User tuteur = chercherUser(utilisateur.numtuteur);
        if (tuteur == null)
            return(false);
        if (emailAdmin == null || "".equals(emailAdmin))
            return(false);
		String subject = titreFormation(utilisateur.formation) + " : " + Strings.get("inscription_de") + " " + utilisateur.prenom + " " + utilisateur.nom;
        String body = Strings.get("bonjour") + "\n";
        body += "\n";
        body += Strings.get("inscription_votre_etudiant") + " \"" + titreFormation(utilisateur.formation) + "\".\n";
        body += "\n";
        body += Strings.get("son_nom") + " " + utilisateur.prenom + " " + utilisateur.nom + ".\n";
        body += Strings.get("son_nom_util") + " " + utilisateur.username + ".\n";
        body += Strings.get("son_courriel") + " " + utilisateur.email + " .\n";
        body += "\n";
        final Mail courriel = new Mail(this, req.getServerName(), smtpServer, 25, emailAdmin, tuteur.email, null);
        courriel.definirMessage(subject, body);
        courriel.envoyer(false);
        return(true);
    }
    
    public void envoyerPageCoordonnees(final HttpServletResponse resp, final Coord coord) {
        try {
            resp.setHeader("Pragma", "no-cache");
            resp.setHeader("Cache-Control", "no-cache");
            resp.setHeader("Expires", "0");
            resp.setContentType("text/html; charset=UTF-8");
            
            final Writer out = resp.getWriter();
            final PrintWriter pw = new PrintWriter(out);
            enteteHTML(pw, Strings.get("admin_edition_coord"));
            menu(pw, null, null);
            pw.println("<h1>" + Strings.get("titre_administration") + "</h1>");
            pw.println("<h2>" + Strings.get("titre_coordonnees") + "</h2>");
            pw.println("<form method=\"post\" action=\"Administration\">");
            pw.println("<input type=\"hidden\" name=\"action\" value=\"modifier_coord\">");
            pw.println("<input type=\"hidden\" name=\"numero\" value=\"" + coord.numero + "\">");
            pw.println("<input type=\"hidden\" name=\"numuser\" value=\"" + coord.numuser + "\">");
            pw.println("<table border=\"1\" cellpadding=\"2\" cellspacing=\"0\">");
            pw.println("<tr><td rowspan=\"2\">" + Strings.get("rue") + "</td>");
            pw.println("<td><input type=\"text\" name=\"rue1\" size=\"40\" maxlength=\"100\" value=\"" +
                entites(coord.rue1) + "\"></td></tr>");
            pw.println("<tr>");
            pw.println("<td><input type=\"text\" name=\"rue2\" size=\"40\" maxlength=\"100\" value=\"" +
                entites(coord.rue2) + "\"></td></tr>");
            pw.println("<tr><td>" + Strings.get("code_postal") + "</td>");
            pw.println("<td><input type=\"text\" name=\"codepostal\" size=\"5\" maxlength=\"5\" value=\"" +
                entites(coord.codepostal) + "\"></td></tr>"); // doit être un entier d'après la base (à faire: vérifier la valeur)
            pw.println("<tr><td>" + Strings.get("ville") + "</td>");
            pw.println("<td><input type=\"text\" name=\"ville\" size=\"30\" maxlength=\"50\" value=\"" +
                entites(coord.ville) + "\"></td></tr>");
            pw.println("<tr><td>" + Strings.get("pays") + "</td>");
            pw.println("<td><input type=\"text\" name=\"pays\" size=\"20\" maxlength=\"30\" value=\"" +
                entites(coord.pays) + "\"></td></tr>");
            pw.println("<tr><td>" + Strings.get("telephone") + "</td>");
            pw.println("<td><input type=\"text\" name=\"telephone\" size=\"20\" maxlength=\"20\" value=\"" +
                entites(coord.telephone) + "\"></td></tr>");
            pw.println("<tr><td>" + Strings.get("connexion") + "</td>");
            pw.println("<td><select name=\"connection\">");
            for (String connection : connectionval) {
                pw.print("  <option value=\"" + connection + "\"");
                if (connection.equals(coord.connection) || ("".equals(connection) && coord.connection == null))
                    pw.print(" selected");
                pw.println(">" + connection + "</option>");
            }
            pw.println("</select></td></tr>");
            pw.println("<tr><td>" + Strings.get("acces") + "</td>");
            pw.println("<td><select name=\"acces\">");
            for (String acces : accesval) {
                pw.print("  <option value=\"" + acces + "\"");
                if (acces.equals(coord.acces) || ("".equals(acces) && coord.acces == null))
                    pw.print(" selected");
                pw.println(">" + acces + "</option>");
            }
            pw.println("</select></td></tr>");
            pw.println("</table>");
            pw.println("<p><input type=\"submit\" value=\"" + Strings.get("enregistrer_champs") + "\"></p>");
            pw.println("</form>");
            
            pw.println("<p><a href=\"Administration?numutilisateur=" + coord.numuser + "\">" + Strings.get("retour_utilisateur") + "</a></p>");
            piedHTML(pw);
        } catch (IOException ex) {
            log("envoyerPageCoordonnees: " + ex.getMessage());
        }
    }
    
    public void modifierCoordonnees(final HttpServletRequest req, final HttpServletResponse resp) {
        String numero = req.getParameter("numero");
        String numuser = req.getParameter("numuser");
        String rue1 = req.getParameter("rue1");
        String rue2 = req.getParameter("rue2");
        String codepostal = req.getParameter("codepostal");
        String ville = req.getParameter("ville");
        String pays = req.getParameter("pays");
        String telephone = req.getParameter("telephone");
        String connection = req.getParameter("connection");
        String acces = req.getParameter("acces");
        Coord coord = chercherCoord(numero);
        if (coord == null) {
            envoyerErreur(resp, Strings.get("coordonnees_introuvables"));
            return;
        }
        // vérification des valeurs
        if (numuser == null || !numuser.equals(coord.numuser)) {
            envoyerErreur(resp, Strings.get("no_util_incorrect_coord") + " " + numuser);
            return;
        }
        if (rue1 == null) {
            envoyerErreur(resp, Strings.get("manque_rue1"));
            return;
        }
        if (rue1.length() > 100) {
            envoyerErreur(resp, Strings.get("rue1_trop_long"));
            return;
        }
        if (rue2 != null && rue2.length() > 100) {
            envoyerErreur(resp, Strings.get("rue2_trop_long"));
            return;
        }
        if (codepostal != null && !"".equals(codepostal)) {
            try {
                Integer n_codepostal = Integer.valueOf(codepostal);
            } catch (NumberFormatException ex) {
                envoyerErreur(resp, Strings.get("cp_entier"));
                return;
            }
        }
        if (ville != null && ville.length() > 50) {
            envoyerErreur(resp, Strings.get("ville_trop_long"));
            return;
        }
        if (pays != null && pays.length() > 30) {
            envoyerErreur(resp, Strings.get("pays_trop_long"));
            return;
        }
        if (telephone != null && telephone.length() > 20) {
            envoyerErreur(resp, Strings.get("telephone_trop_long"));
            return;
        }
        if (connection != null && !"".equals(connection)) {
            final boolean trouve = Arrays.asList(connectionval).contains(connection);
            if (!trouve) {
                envoyerErreur(resp, Strings.get("connexion_inconnue") + " " + connection);
                return;
            }
        }
        if (acces != null && !"".equals(acces)) {
            final boolean trouve = Arrays.asList(accesval).contains(acces);
            if (!trouve) {
                envoyerErreur(resp, Strings.get("acces_inconnu") + " " + acces);
                return;
            }
        }
        coord.rue1 = rue1;
        coord.rue2 = rue2;
        if ("".equals(codepostal))
            codepostal = null;
        coord.codepostal = codepostal;
        coord.ville = ville;
        coord.pays = pays;
        coord.telephone = telephone;
        coord.connection = connection;
        coord.acces = acces;
        try {
            coordDB.modCoord(coord);
        } catch (JDBException ex) {
            log("Administration: modifierCoordonnees: JDBException: " + ex.getMessage());
            envoyerErreur(resp, Strings.get("erreur_modif_coord"));
            return;
        }
        try {
            resp.setContentType("text/html; charset=UTF-8");
            final Writer out = resp.getWriter();
            final PrintWriter pw = new PrintWriter(out);
            enteteHTML(pw, Strings.get("admin_edition_coord"));
            menu(pw, null, null);
            pw.println("<h1>" + Strings.get("titre_administration") + "</h1>");
            pw.println("<h2>" + Strings.get("titre_coordonnees") + "</h2>");
            pw.println("<p>" + Strings.get("modifs_enregistrees") + "</p>");
            pw.println("<p><a href=\"Administration?numutilisateur=" + numuser + "\">" + Strings.get("retour_utilisateur") + "</a></p>");
            piedHTML(pw);
        } catch (IOException ex) {
            log("Administration: modifierCoordonnees: " + ex.getMessage());
        }
    }
    
    public void fermerCompte(final HttpServletRequest req, final HttpServletResponse resp) {
        final String numutilisateur = req.getParameter("numutilisateur");
        final String comptes = req.getParameter("comptes");
        final String tri = req.getParameter("tri");
        final User utilisateur = chercherUser(numutilisateur);
        if (utilisateur == null) {
            log("Administration: fermerCompte: " + Strings.get("erreur_no_utilisateur") + ": " + numutilisateur);
            envoyerErreur(resp, Strings.get("erreur_no_utilisateur"));
            return;
        }
        final Date now = new Date();
        final SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        final String date = formatter.format(now);
        utilisateur.datefermeture = date;
        try {
            userDB.modUser(utilisateur);
        } catch (JDBException ex) {
            log("Administration: fermerCompte: JDBException: " + ex.getMessage());
            envoyerErreur(resp, Strings.get("erreur_enregistrement_fermeture"));
            return;
        }
        envoyerListeUtilisateurs(resp, comptes, tri);
    }
    
    public void supprimerUtilisateur(final HttpServletRequest req, final HttpServletResponse resp) {
        final String numutilisateur = req.getParameter("numutilisateur");
        final String confirmation = req.getParameter("confirmation");
        final String page = req.getParameter("page");
        final String comptes = req.getParameter("comptes");
        final String tri = req.getParameter("tri");
        final User utilisateur = chercherUser(numutilisateur);
        if (utilisateur == null) {
            log("Administration: supprimerUtilisateur: " + Strings.get("erreur_no_utilisateur") + ": " + numutilisateur);
            envoyerErreur(resp, Strings.get("erreur_no_utilisateur"));
            return;
        }
        if (Strings.get("annuler").equals(confirmation)) {
            if ("inscriptions".equals(page))
                envoyerListeInscriptions(resp);
            else
                envoyerListeUtilisateurs(resp, comptes, tri);
            return;
        }
        if (!Strings.get("oui").equals(confirmation)) {
            try {
                resp.setHeader("Pragma", "no-cache");
                resp.setHeader("Cache-Control", "no-cache");
                resp.setHeader("Expires", "0");
                resp.setContentType("text/html; charset=UTF-8");
                final PrintWriter pw = resp.getWriter();
                enteteHTML(pw, Strings.get("suppression_util"));
                final String liste;
                if ("tous".equals(comptes))
                    liste = "tous";
                else
                    liste = page;
                menu(pw, liste, tri);
                pw.println("<h1>" + Strings.get("titre_administration") + "</h1>");
                pw.println("<h2>" + Strings.get("suppression_util") + "</h2>");
                pw.println("<p>" + Strings.get("confirmation_suppression_util_1") + " " + utilisateur.username + " (" +
                    utilisateur.prenom + " " + utilisateur.nom + ") " + Strings.get("confirmation_suppression_util_2") + "</p>");
                pw.println("<form method=\"post\" action=\"Administration\">");
                pw.println("<input type=\"hidden\" name=\"action\" value=\"supprimer_util\">");
                pw.println("<input type=\"hidden\" name=\"numutilisateur\" value=\"" + numutilisateur + "\">");
                if (page != null)
                    pw.println("<input type=\"hidden\" name=\"page\" value=\"" + page + "\">");
                if (comptes != null)
                    pw.println("<input type=\"hidden\" name=\"comptes\" value=\"" + comptes + "\">");
                if (tri != null)
                    pw.println("<input type=\"hidden\" name=\"tri\" value=\"" + tri + "\">");
                pw.println("<p><input type=\"submit\" name=\"confirmation\" value=\"" + Strings.get("annuler") + "\">");
                pw.println("<input type=\"submit\" name=\"confirmation\" value=\"" + Strings.get("oui") + "\"></p>");
                pw.println("</form>");
                piedHTML(pw);
            } catch (IOException ex) {
                log("Administration: supprimerUtilisateur: " + ex.getMessage());
            }
            return;
        }
        try {
            if (connDB == null)
                connDB = new ConnectionDB(userDBURL, userDBUSER, userDBPASSWORD);
            if (lectDB == null)
                lectDB = new LectureDB(userDBURL, userDBUSER, userDBPASSWORD);
            if (connDB.tableExists()) {
                Connection findconn = new Connection(null, utilisateur.numero, null, null);
                final Vector<Connection> connections = connDB.findConnections(findconn);
                for (Connection conn : connections) {
                    final Lecture findlect = new Lecture(null, null, conn.numero);
                    final Vector<Lecture> lectures = lectDB.findLectures(findlect);
                    for (Lecture lect : lectures)
                        lectDB.delLecture(lect);
                    connDB.delConnection(conn);
                }
            }
            if (exoDB == null)
                exoDB = new ExerciceDB(userDBURL, userDBUSER, userDBPASSWORD);
            if (exoDB.tableExists()) {
                final Exercice findexo = new Exercice(utilisateur.numero);
                final Vector<Exercice> exercices = exoDB.findExercices(findexo);
                for (Exercice exo : exercices)
                    exoDB.delExercice(exo);
            }
            final Coord findcoord = new Coord(utilisateur.numero);
            final Vector<Coord> coordonnees = coordDB.findCoords(findcoord);
            for (Coord coord : coordonnees)
                coordDB.delCoord(coord);
            userDB.delUser(utilisateur);
        } catch (JDBException ex) {
            log("Administration: supprimerUtilisateur: JDBException: " + ex.getMessage());
            envoyerErreur(resp, Strings.get("erreur_suppression_util"));
            return;
        }
        if ("inscriptions".equals(page))
            envoyerListeInscriptions(resp);
        else
            envoyerListeUtilisateurs(resp, comptes, tri);
    }
    
    public void envoyerPageGenerationSite(final HttpServletResponse resp) {
        try {
            resp.setHeader("Pragma", "no-cache");
            resp.setHeader("Cache-Control", "no-cache");
            resp.setHeader("Expires", "0");
            resp.setContentType("text/html; charset=UTF-8");
            
            final Writer out = resp.getWriter();
            final PrintWriter pw = new PrintWriter(out);
            enteteHTML(pw, Strings.get("generation_site"));
            menu(pw, "generation", null);
            pw.println("<h1>" + Strings.get("generation_site") + "</h1>");
            pw.println("<h2>" + Strings.get("creation_web_formation") + "</h2>");
            pw.println("<p>" + Strings.get("infos_creation") + "</p>");
            if ((new File(cheminXMLSite)).exists()) {
                pw.println("<form method=\"post\" action=\"Administration\">");
                pw.println("<input type=\"hidden\" name=\"action\" value=\"generer_site\">");
                pw.println("<p><input type=\"submit\" value=\"" + Strings.get("generer_formation") + "\"></p>");
                pw.println("</form>");
                if ((new File(baseSitePublic)).getParentFile().exists()) {
                    pw.println("<h2>" + Strings.get("creation_web_public") + "</h2>");
                    pw.println("<form method=\"post\" action=\"Administration\">");
                    pw.println("<input type=\"hidden\" name=\"action\" value=\"generer_site_public\">");
                    pw.println("<p><input type=\"submit\" value=\"" + Strings.get("generer_public") + "\"></p>");
                    pw.println("</form>");
                }
            } else
                pw.println("<p>" + Strings.get("XML_introuvable") + "</p>");
            
            piedHTML(pw);
        } catch (IOException ex) {
            log("envoyerPageCoordonnees: " + ex.getMessage());
        }
    }
    
    public void genererSite(final HttpServletRequest req, final HttpServletResponse resp, final boolean sitePublic) {
        PrintWriter pw = null;
        try {
            resp.setHeader("Pragma", "no-cache");
            resp.setHeader("Cache-Control", "no-cache");
            resp.setHeader("Expires", "0");
            resp.setContentType("text/html; charset=UTF-8");
            
            final Writer out = resp.getWriter();
            pw = new PrintWriter(out);
            enteteHTML(pw, Strings.get("generation_site"));
            menu(pw, "generation", null);
            pw.println("<h1>" + Strings.get("generation_site") + "</h1>");
        } catch (IOException ex) {
            log("genererSite: " + ex.getMessage());
        }
        
        // transformations XSLT
        final File fichierXML = new File(cheminXMLSite);
        final HashMap<String,String> parametresXSL = new HashMap<String,String>();
        File fichierToutLeCours = null; // sortie XML de la transformation XPAGES1.xsl, sert à XPAGES2 et à l'indexation
        File fichierSiteEnPage = null; // sortie XML de la transformation XPAGES2.xsl, sert à XPAGES3
        File fichierImages = null; // fichier images.xml issu de la transformation XPAGES2.xsl, à effacer après XPAGES3.xsl
        File fichierFinal = null; // sortie HTML directe de la transformation XPAGES3_PAELLOS.xsl, à effacer
        File dossierSiteTmp = null; // dossier temporaire du site pendant la génération
        File dossierSiteTuteursTmp = null; // dossier temporaire du site tuteurs pendant la génération
        final TransformerFactory tFactory = TransformerFactory.newInstance();
        final File fichierXPAGES1 = new File(getServletContext().getRealPath("/WEB-INF/config/" + "XPAGES1.xsl"));
        final File fichierXPAGES2 = new File(getServletContext().getRealPath("/WEB-INF/config/" + "XPAGES2.xsl"));
        final File fichierXPAGES3 = new File(getServletContext().getRealPath("/WEB-INF/config/" + "XPAGES3_PAELLOS.xsl"));
        
        try {
            // création du dossier temporaire du site
            dossierSiteTmp = File.createTempFile("tmpsite", Long.toString(System.nanoTime()));
            if (!dossierSiteTmp.delete())
                throw new IOException("Impossible d'effacer le fichier temporaire: " + dossierSiteTmp.getAbsolutePath());
            if (!dossierSiteTmp.mkdir())
                throw new IOException("Impossible de créer le dossier temporaire: " + dossierSiteTmp.getAbsolutePath());
            
            // XPAGES1.xsl
            pw.println("<p>" + Strings.get("rassemblement_contributions") + "</p>");
            pw.flush(); // remarque: flush ne marche pas avec un connecteur AJP et Tomcat 6
            parametresXSL.put("jaxe-uri-xml", fichierXML.toURI().toASCIIString());
            fichierToutLeCours = java.io.File.createTempFile("tmp", ".xml") ;
            transformationXSLT(tFactory, fichierXPAGES1, parametresXSL, fichierXML, fichierToutLeCours);
            
            // XPAGES2.xsl (site public ou étudiants)
            pw.println("<p>" + Strings.get("copie_redimensionnement") + "</p>");
            pw.flush();
            fichierFinal = new File(dossierSiteTmp, "out.html");
            fichierImages = new File(dossierSiteTmp, "images.xml");
            
            parametresXSL.clear();
            parametresXSL.put("jaxe-fichier-xml", fichierXML.getAbsolutePath());
            parametresXSL.put("jaxe-fichier-xsl", fichierXPAGES2.getAbsolutePath());
            parametresXSL.put("jaxe-fichier-destination", fichierFinal.getAbsolutePath());
            parametresXSL.put("jaxe-uri-destination", fichierFinal.toURI().toASCIIString());
            parametresXSL.put("chemin-site", fichierFinal.getParentFile().getAbsolutePath());
            fichierSiteEnPage = java.io.File.createTempFile("tmp", ".xml") ;
            transformationXSLT(tFactory, fichierXPAGES2, parametresXSL, fichierToutLeCours, fichierSiteEnPage);
            
            // XPAGES3_PAELLOS.xsl (site public ou étudiants)
            pw.println("<p>" + Strings.get("creation_html") + "</p>");
            pw.flush();
            parametresXSL.clear();
            parametresXSL.put("jaxe-fichier-xml", fichierXML.getAbsolutePath());
            parametresXSL.put("jaxe-uri-xsl", fichierXPAGES3.toURI().toASCIIString());
            parametresXSL.put("jaxe-uri-xml", fichierXML.toURI().toASCIIString());
            parametresXSL.put("jaxe-fichier-destination", fichierFinal.getAbsolutePath());
            parametresXSL.put("jaxe-uri-destination", fichierFinal.toURI().toASCIIString());
            parametresXSL.put("chemin-site", fichierFinal.getParentFile().getAbsolutePath());
            parametresXSL.put("uri-site", fichierFinal.getParentFile().toURI().toASCIIString());
            parametresXSL.put("affichage", "web");
            if (sitePublic)
                parametresXSL.put("sortie", "public");
            else
                parametresXSL.put("sortie", "formation");
            transformationXSLT(tFactory, fichierXPAGES3, parametresXSL, fichierSiteEnPage, fichierFinal);
            
            // nettoyage
            if (fichierSiteEnPage.exists())
                fichierSiteEnPage.delete();
            if (fichierImages.exists())
                fichierImages.delete();
            if (fichierFinal.exists())
                fichierFinal.delete();
            
            // remplacement du vrai site par le site temporaire, en conservant l'ancien sous un autre nom
            File dossierSiteFinal;
            if (sitePublic)
                dossierSiteFinal = new File(baseSitePublic);
            else
                dossierSiteFinal = new File(baseSiteEtudiants);
            if (dossierSiteFinal.exists()) {
                final File ancienSite = File.createTempFile(dossierSiteFinal.getName(), "old", dossierSiteFinal.getParentFile());
                pw.println("<p>" + Strings.get("ancien_site_renomme") + " " + ancienSite.getName() + "</p>");
                pw.flush();
                if (!ancienSite.delete())
                    throw new IOException("Impossible d'effacer le fichier temporaire: " + ancienSite.getAbsolutePath());
                if (!dossierSiteFinal.renameTo(ancienSite))
                    throw new IOException("Impossible de déplacer le dossier de l'ancien site: " + dossierSiteFinal.getAbsolutePath() + " -> " + ancienSite.getAbsolutePath());
            }
            if (!dossierSiteTmp.renameTo(dossierSiteFinal))
                throw new IOException("Impossible de déplacer le dossier temporaire: " + dossierSiteTmp.getAbsolutePath() + " -> " + dossierSiteFinal.getAbsolutePath());
            
            if (!sitePublic) {
                pw.println("<p>" + Strings.get("generation_site_tuteurs") + "</p>");
                pw.flush();
                // création du site tuteurs temporaire
                dossierSiteTuteursTmp = File.createTempFile("tmptuteurs", Long.toString(System.nanoTime()));
                if (!dossierSiteTuteursTmp.delete())
                    throw new IOException("Impossible d'effacer le fichier temporaire: " + dossierSiteTuteursTmp.getAbsolutePath());
                if (!dossierSiteTuteursTmp.mkdir())
                    throw new IOException("Impossible de créer le dossier temporaire: " + dossierSiteTuteursTmp.getAbsolutePath());
                
                // XPAGES2.xsl (site tuteurs)
                pw.println("<p>" + Strings.get("copie_redimensionnement") + "</p>");
                pw.flush();
                fichierFinal = new File(dossierSiteTuteursTmp, "out.html");
                fichierImages = new File(dossierSiteTuteursTmp, "images.xml");
                parametresXSL.clear();
                parametresXSL.put("jaxe-fichier-xml", fichierXML.getAbsolutePath());
                parametresXSL.put("jaxe-fichier-xsl", fichierXPAGES2.getAbsolutePath());
                parametresXSL.put("jaxe-fichier-destination", fichierFinal.getAbsolutePath());
                parametresXSL.put("jaxe-uri-destination", fichierFinal.toURI().toASCIIString());
                parametresXSL.put("chemin-site", fichierFinal.getParentFile().getAbsolutePath());
                fichierSiteEnPage = java.io.File.createTempFile("tmp", ".xml") ;
                transformationXSLT(tFactory, fichierXPAGES2, parametresXSL, fichierToutLeCours, fichierSiteEnPage);
                
                // XPAGES3_PAELLOS.xsl (site tuteurs)
                pw.println("<p>" + Strings.get("creation_html") + "</p>");
                pw.flush();
                parametresXSL.clear();
                parametresXSL.put("jaxe-fichier-xml", fichierXML.getAbsolutePath());
                parametresXSL.put("jaxe-uri-xsl", fichierXPAGES3.toURI().toASCIIString());
                parametresXSL.put("jaxe-uri-xml", fichierXML.toURI().toASCIIString());
                parametresXSL.put("jaxe-fichier-destination", fichierFinal.getAbsolutePath());
                parametresXSL.put("jaxe-uri-destination", fichierFinal.toURI().toASCIIString());
                parametresXSL.put("chemin-site", fichierFinal.getParentFile().getAbsolutePath());
                parametresXSL.put("uri-site", fichierFinal.getParentFile().toURI().toASCIIString());
                parametresXSL.put("affichage", "web");
                parametresXSL.put("sortie", "tuteurs");
                transformationXSLT(tFactory, fichierXPAGES3, parametresXSL, fichierSiteEnPage, fichierFinal);
                
                // nettoyage
                if (fichierSiteEnPage.exists())
                    fichierSiteEnPage.delete();
                if (fichierImages.exists())
                    fichierImages.delete();
                if (fichierFinal.exists())
                    fichierFinal.delete();
                
                // remplacement du vrai site tuteurs
                final File dossierSiteTuteurs = new File(baseSiteTuteurs);
                if (dossierSiteTuteurs.exists()) {
                    final File ancienSite = File.createTempFile(dossierSiteTuteurs.getName(), "old", dossierSiteTuteurs.getParentFile());
                    pw.println("<p>" + Strings.get("ancien_site_renomme") + " " + ancienSite.getName() + "</p>");
                    pw.flush();
                    if (!ancienSite.delete())
                        throw new IOException("Impossible d'effacer le fichier temporaire: " + ancienSite.getAbsolutePath());
                    if (!dossierSiteTuteurs.renameTo(ancienSite))
                        throw new IOException("Impossible de déplacer le dossier de l'ancien site: " + dossierSiteTuteurs.getAbsolutePath() + " -> " + ancienSite.getAbsolutePath());
                }
                if (!dossierSiteTuteursTmp.renameTo(dossierSiteTuteurs))
                    throw new IOException("Impossible de déplacer le dossier temporaire: " + dossierSiteTuteursTmp.getAbsolutePath() + " -> " + dossierSiteTuteurs.getAbsolutePath());
                
                // indexation
                final IndexXPAGES index = new IndexXPAGES(indexLucene, fichierToutLeCours);
                index.indexation();
                fichierToutLeCours.delete();
            }
            
        } catch (Exception ex) {
            log("genererSite: " + ex.getClass().getName() + ": " + ex.getMessage());
            if (fichierToutLeCours != null && fichierToutLeCours.exists())
                fichierToutLeCours.delete();
            if (fichierSiteEnPage != null && fichierSiteEnPage.exists())
                fichierSiteEnPage.delete();
            if (dossierSiteTmp != null && dossierSiteTmp.exists()) {
                try {
                    effacerRecursivement(dossierSiteTmp);
                } catch (IOException ex2) {
                    log("genererSite: effacerRecursivement: " + ex2.getClass().getName() + ": " + ex2.getMessage());
                }
            }
            if (dossierSiteTuteursTmp != null && dossierSiteTuteursTmp.exists()) {
                try {
                    effacerRecursivement(dossierSiteTuteursTmp);
                } catch (IOException ex2) {
                    log("genererSite: effacerRecursivement: " + ex2.getClass().getName() + ": " + ex2.getMessage());
                }
            }
            pw.println("<p>" + Strings.get("une_erreur") + " " + ex.getMessage() + "</p>");
            piedHTML(pw);
            return;
        }
        
        // résultat: OK
        pw.println("<p>" + Strings.get("site_genere_sans_erreur") + "</p>");
        
        piedHTML(pw);
    }
    
    public void transformationXSLT(final TransformerFactory tFactory, final File fichierXSL, final HashMap<String,String> parametres,
            final File fichierEntree, final File fichierSortie) throws Exception {
        try {
            final Transformer transformer = tFactory.newTransformer(new StreamSource(fichierXSL));
            for (Map.Entry<String,String> parametre : parametres.entrySet())
                transformer.setParameter(parametre.getKey(), parametre.getValue());
            final StreamSource source = new StreamSource(fichierEntree);
            final StreamResult result = new StreamResult(fichierSortie);
            transformer.transform(source, result);
        } catch (final TransformerException ex) {
            SourceLocator loc = null;
            Throwable cause = ex;
            do {
                if (cause instanceof SAXParseException)
                    loc = new SAXSourceLocator((SAXParseException)cause);
                else if (cause instanceof TransformerException) {
                    final SourceLocator loc2 = ((TransformerException)cause).getLocator();
                    if (loc2 != null)
                        loc = loc2;
                }
                if (cause instanceof TransformerException)
                    cause = ((TransformerException)cause).getCause();
                else if (cause instanceof WrappedRuntimeException)
                    cause = ((WrappedRuntimeException)cause).getException();
                else if (cause instanceof SAXException)
                    cause = ((SAXException)cause).getException();
                else
                    cause = null;
            } while (cause != null);
            String infos = "XSLT: " + fichierXSL.getName() + ": " + ex.getClass().getName() + ": " + ex.getMessage();
            if (loc != null)
                infos += " at line " + loc.getLineNumber();
            throw new Exception(infos, ex);
        }
    }
    
    public void deconnexion(final HttpSession session, final HttpServletResponse resp) {
        session.invalidate();
        try {
            resp.setHeader("Pragma", "no-cache");
            resp.setHeader("Cache-Control", "no-cache");
            resp.setHeader("Expires", "0");
            resp.setContentType("text/html; charset=UTF-8");
            
            final PrintWriter pw = resp.getWriter();
            pw.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
            pw.println("<html>");
            pw.println("<head>");
            pw.println(" <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">");
            pw.println(" <meta http-equiv=\"Refresh\" content=\"3; url=" + URLServeur + pageLogin + "\">");
            pw.println(" <title>" + Strings.get("deconnexion") + "</title>");
            pw.println(" <link href=\"paellos.css\" rel=\"stylesheet\" type=\"text/css\">");
            pw.println("</head>");
            pw.println("<body>");
            pw.println("<p>" + Strings.get("vous_deconnecte") + "</p>");
            pw.println("</body>");
            pw.println("</html>");
            pw.flush();
        } catch (IOException ex) {
            log("Administration: deconnexion: " + ex.getMessage());
        }
    }
    
	/**
	  * Efface un dossier et son contenu récursivement
	  */
	public static void effacerRecursivement(final File d) throws IOException {
	   if (d.isDirectory()) {
	       for (File f : d.listFiles())
	           effacerRecursivement(f);
	   }
		if (!d.delete())
			throw new IOException("Impossible d'effacer : " + d.getPath());
	}
	
    // encodage d'entités HTML
    public static String entites(final String s) {
        if (s == null)
            return("");
        String res = "";
        for (int i=0; i<s.length(); i++) {
            char c = s.charAt(i);
            if (c >= 'a' && c <= 'z' || c >= 'A' && c <= 'Z' || c >= '0' && c <= '9')
                res += c;
            else if (" .,;:-_=/*+()@?!%".indexOf(c) != -1)
                res += c;
            else if ("àéèêëïîôùûç".indexOf(c) != -1)
                res += c;
            else if (c == '"')
                res += "&quot;";
            else if (c == '&')
                res += "&amp;";
            else if (c == '<')
                res += "&lt;";
            else if (c == '>')
                res += "&gt;";
            else if (Character.isWhitespace(c) || !Character.isISOControl(c))
                res += "&#" + (int)c + ";";
        }
        return(res);
    }
    
}
