/*
PAELLOS - Plateforme d'Apprentissage En Ligne Libre Orientée Sémantique

Copyright (C) 2011 Observatoire de Paris

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package paellos;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Transparency;
import java.awt.color.ColorSpace;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.ComponentColorModel;
import java.awt.image.DataBuffer;
import java.awt.image.WritableRaster;
import java.io.IOException;

import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageTypeSpecifier;
import javax.imageio.ImageWriter;
import javax.imageio.metadata.IIOInvalidTreeException;
import javax.imageio.metadata.IIOMetadata;
import javax.imageio.metadata.IIOMetadataNode;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import jaxe.equations.MathBase;
import jaxe.equations.StringMathBuilder;


/**
 * Si on utilise directement ces méthodes sur le serveur depuis Jaxe.jar, on a l'erreur :
 * java.lang.InternalError: Can't connect to window server - not enough permissions.
 * parce-que DialogueEquation est dérivé de Component.
 * -> on évite Component en sortant ces méthodes de DialogueEquation
 * ... mais en fait on a toujours l'erreur, cette fois quand on crée un BufferedImage.
 * ... essai avec l'option de Java -Djava.awt.headless=true dans catalina.sh
 * (de toutes façons il faut éviter Dialog avec headless)
 *
 * @author Damien Guillaume
 */
class Equation {
    
    /**
     * Création de l'image correspondant au texte de l'équation
     */
    public static BufferedImage creerImage(final String texteEquation) {
        // reconstruction de l'image
        final BufferedImage imgcalc = new BufferedImage(10, 10, BufferedImage.TYPE_INT_ARGB);
        final Graphics2D gcalc = imgcalc.createGraphics();
        // on est obligé de créer un Graphics bidon pour calculer la taille de l'image depuis que la méthode
        // Toolkit.getFontMetrics(Font) est dépréciée
        final MathBase base = new MathBase((new StringMathBuilder(texteEquation)).getMathRootElement(), gcalc);
        final Dimension dim = new Dimension(base.getWidth(), base.getHeight());
        
        // IE ne gère pas les PNG avec une palette -> utilisation d'une échelle de gris avec transparence
        final ColorSpace grayColorSpace = ColorSpace.getInstance(ColorSpace.CS_GRAY);
        final ColorModel cm = new ComponentColorModel(grayColorSpace, true, true, Transparency.TRANSLUCENT, DataBuffer.TYPE_BYTE);
        final WritableRaster raster = cm.createCompatibleWritableRaster(dim.width, dim.height);
        final BufferedImage img2 = new BufferedImage(cm, raster, false, null);
        final Graphics2D g2 = img2.createGraphics();
        g2.setColor(Color.black);
        base.paint(g2);
        gcalc.dispose();
        g2.dispose();
        
        return img2;
    }
    
    /**
     * Enregistrement de l'image de l'équation vers une sortie donnée en paramètre.
     * Le paramètre sortie peut être de la classe File (pour enregistrer dans un fichier) ou OutputStream.
     * Les images peuvent être utilisées dans IE6 (PNG avec une échelle de gris et fond blanc, couche alpha
     * utilisable par les autres navigateurs).
     */
    public static void enregistrerImage(final BufferedImage img, final Object sortie) throws IOException {
        // IE met un fond gris si aucun bKGD n'est spécifié -> ajout de bKGD dans les métadonnées
        final ImageWriter premierPourPNG = ImageIO.getImageWritersByFormatName("PNG").next();
        final IIOMetadata metadata = premierPourPNG.getDefaultImageMetadata(new ImageTypeSpecifier(img), null);
        final Element racine = (Element)metadata.getAsTree("javax_imageio_png_1.0");
        final NodeList nl = racine.getElementsByTagName("bKGD");
        if (nl.getLength() == 0) {
            Node suivant = racine.getFirstChild();
            final String name = suivant.getNodeName();
            if (("IHDR".equals(name) || "PLTE".equals(name)))
                suivant = suivant.getNextSibling();
            if (suivant != null && "PLTE".equals(suivant.getNodeName()))
                suivant = suivant.getNextSibling();
            final Element bKGD = new IIOMetadataNode("bKGD");
            if (suivant == null)
                racine.appendChild(bKGD);
            else
                racine.insertBefore(bKGD, suivant);
            final Element bKGD_Grayscale = new IIOMetadataNode("bKGD_Grayscale");
            bKGD.appendChild(bKGD_Grayscale);
            bKGD_Grayscale.setAttribute("gray", "255");
            try {
                metadata.mergeTree("javax_imageio_png_1.0", racine);
            } catch (final IIOInvalidTreeException ex) {
                System.err.println("enregistrerImage(BufferedImage, Object): " +  ex.getClass() + ": " + ex.getMessage());
            }
        }
        // enregistrement (peut lancer une IOException)
        premierPourPNG.setOutput(ImageIO.createImageOutputStream(sortie));
        premierPourPNG.write(new IIOImage(img, null, metadata));
    }
    
}
