/*
PAELLOS - Plateforme d'Apprentissage En Ligne Libre Orientée Sémantique

Copyright (C) 2011 Observatoire de Paris

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package paellos;

import org.apache.lucene.analysis.*;
import org.apache.lucene.analysis.standard.StandardTokenizer;
import org.apache.lucene.analysis.standard.StandardFilter;
import java.io.Reader;
import java.util.Hashtable;

public class FrenchAnalyzer extends Analyzer {

    private Hashtable stopTable;
    
    public static final String[] STOP_WORDS = {
        "à", "au", "aux", "ce", "ces", "cet", "cette", "ceux",
        "dans", "de", "des", "donc", "dont", "du",
        "dès", "en", "et", "il", "ils", "je",
        "la", "le", "les", "ma", "me", "mes", "ne",
        "ni", "non", "on", "ou", "où", "par",
        "que", "qui", "sa", "se", "ses", "sur",
        "te", "tes", "ton", "tu", "un", "une",
        "y", "ça"
    };
    
    /** Builds an analyzer. */
    public FrenchAnalyzer() {
        this(STOP_WORDS);
    }
    
    /** Builds an analyzer with the given stop words. */
    public FrenchAnalyzer(String[] stopWords) {
        stopTable = StopFilter.makeStopTable(stopWords);
    }

    public final TokenStream tokenStream(String fieldName, Reader reader) {
        TokenStream result = new StandardTokenizer(reader);
        //result = new FrenchStemFilter(result);  marche pas :(
        result = new StandardFilter(result);
        result = new LowerCaseFilter(result);
        result = new StopFilter(result, stopTable);
        return result;
    }
}
