/*
PAELLOS - Plateforme d'Apprentissage En Ligne Libre Orientée Sémantique

Copyright (C) 2011 Observatoire de Paris

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/
package paellos;

import java.io.File;
import java.io.IOException;
import java.util.Date;

import org.xml.sax.SAXException;

import org.apache.lucene.index.*;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;

/**
  * création de l'index à partir du fichier XPAGES contenant tout le cours (sans référence)
  */
class IndexXPAGES {
    private String index;
    private File fichierToutLeCours;
    private IndexWriter writer;
    private int ndocs = 0;
    
    /**
     * index: chemin du dossier de l'index Lucene
     * fichierToutLeCours: fichier XML XPAGES du site
     */
    public IndexXPAGES(final String index, final File fichierToutLeCours) {
        this.index = index;
        this.fichierToutLeCours = fichierToutLeCours;
    }
    
    public void indexation() throws IOException, SAXException {
        Date start = new Date();
        
        writer = new IndexWriter(index, new FrenchAnalyzer(), true);
        writer.maxFieldLength = 1000000;
        
        indexDocs(fichierToutLeCours);
        
        //System.out.println(ndocs + " documents");
        
        //System.out.println("Optimizing index...");
        writer.optimize();
        writer.close();
        
        Date end = new Date();
        
        //System.out.print(end.getTime() - start.getTime());
        //System.out.println(" total milliseconds");
    }
    
    private void indexDocs(final File toutlecours) throws IOException, SAXException {
        XPAGESParser parser = new XPAGESParser(this, toutlecours);
        parser.parse();
    }
    
    public void ajouterDocument(final String nom, final String titre, final String contenu) {
        if (nom == null || contenu == null)
            return;
        //if (nom.lastIndexOf('/') != -1)
        //    System.out.println(nom.substring(nom.lastIndexOf('/')));
        Document doc = new Document();
        doc.add(Field.Keyword("fichier", nom));
        if (titre != null)
            doc.add(Field.Text("title", titre));
        doc.add(Field.Text("contents", contenu));
        try {
            writer.addDocument(doc);
        } catch (IOException ex) {
            System.err.println("ajouterDocument: " + ex.getClass().getName() + ": " + ex.getMessage());
        }
        ndocs++;
    }
}
