/*
PAELLOS - Plateforme d'Apprentissage En Ligne Libre Orientée Sémantique

Copyright (C) 2011 Observatoire de Paris

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package paellos;

import java.awt.image.BufferedImage;
import java.io.*;
import java.security.NoSuchAlgorithmException;
import java.util.*;

import java.sql.SQLException;
import javax.sql.rowset.serial.SerialBlob;
import javax.sql.rowset.serial.SerialException;

import java.text.ParseException;
import java.text.SimpleDateFormat;

import javax.servlet.*;
import javax.servlet.http.*;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.xml.sax.InputSource;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;

import paellos.jdb.*;
import paellos.base.*;


/**
 * Servlet pour CELA: login, lecture des fichiers, réponses aux exercices, changement de mot de passe.
 *
 * @author Damien Guillaume
 */
public class Site extends HttpServlet {

    // valeurs dans le fichier ServletProperties
    String baseSiteEtudiants;
    String baseSiteTuteurs;
    String URLServeur;
    String pageLogin;
    String cheminSiteEtudiants;
    String cheminSiteTuteurs;
    String smtpServer;
    String emailAdmin; // le serveur SMTP doit accepter cet email dans le "From:"
    protected String userDBURL;
    protected String userDBUSER;
    protected String userDBPASSWORD;
    protected String[] noms_formations;
    protected String[] titres_formations;
    
    UserDB userDB;
    LectureDB lectDB;
    ConnectionDB connDB;
    ExerciceDB exoDB;
    
    Transformer reponsesXSLT;
    
    
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
        final Properties props = new Properties();
        try {
            props.load(new InputStreamReader(getClass().getResourceAsStream("ServletProperties"), "UTF-8"));
            // autre option, avec fichier ailleurs: 
    // props.load(getSexrvletConfig().getServletContext().getResourceAsStream("/WEB-INF/properties/bar.properties"))
            baseSiteEtudiants = props.getProperty("baseSiteEtudiants");
            baseSiteTuteurs = props.getProperty("baseSiteTuteurs");
            URLServeur = props.getProperty("URLServeur");
            pageLogin = props.getProperty("pageLogin");
            cheminSiteEtudiants = props.getProperty("cheminSiteEtudiants");
            cheminSiteTuteurs = props.getProperty("cheminSiteTuteurs");
            smtpServer = props.getProperty("smtpServer");
            emailAdmin = props.getProperty("emailAdmin");
            userDBURL = props.getProperty("userDBURL");
            userDBUSER = props.getProperty("userDBUSER");
            userDBPASSWORD = props.getProperty("userDBPASSWORD");
            String s_formations = props.getProperty("formations");
            if (s_formations == null || "".equals(s_formations)) {
                noms_formations = null;
                titres_formations = null;
            } else {
                String[] formations = s_formations.split("\\|");
                noms_formations = new String[formations.length];
                titres_formations = new String[formations.length];
                for (int i=0; i<formations.length; i++) {
                    String[] nom_titre = formations[i].split(":");
                    noms_formations[i] = nom_titre[0];
                    if (nom_titre.length > 0)
                        titres_formations[i] = nom_titre[1];
                    else
                        titres_formations[i] = noms_formations[i];
                }
            }
        } catch (IOException ex) {
            log(Strings.get("erreur_proprietes") + " " + ex.getMessage());
        }
        userDB = new UserDB(userDBURL, userDBUSER, userDBPASSWORD);
        connDB = new ConnectionDB(userDBURL, userDBUSER, userDBPASSWORD);
        lectDB = new LectureDB(userDBURL, userDBUSER, userDBPASSWORD);
        exoDB = new ExerciceDB(userDBURL, userDBUSER, userDBPASSWORD);
        try {
            if (!connDB.tableExists()) {
                log(Strings.get("info_creation_table_connections"));
                connDB.createDB();
            }
            if (!lectDB.tableExists()) {
                log(Strings.get("info_creation_table_lectures"));
                lectDB.createDB();
            }
            if (!exoDB.tableExists()) {
                log(Strings.get("info_creation_table_exercices"));
                exoDB.createDB();
            }
        } catch (JDBException ex) {
            log("Erreur: tableExists/createDB: " + ex.getMessage());
        }
        try {
            final TransformerFactory tFactory = TransformerFactory.newInstance() ;
            reponsesXSLT = tFactory.newTransformer(new StreamSource(getClass().getResourceAsStream("REPONSES.xsl")));
        } catch (Exception ex) {
            log("init Site: Création du transformer XSLT pour les réponses: " + ex.getClass().getName() + ": " + ex.getMessage());
        }
    }
    
    
    /**
     * Paramètres possibles pour une requête POST :
     * POUR UNE CONNEXION AU SITE :
     * - login : login pour la connexion au site
     * - password : mot de passe pour la connexion au site
     * - redirection : page où rediriger si l'authentification est réussie
     *
     * POUR UNE REPONSE A UN EXERCICE (la requête doit être de type multipart/form-data) :
     * - titreexo : titre de l'exercice
     * - cheminpage : chemin de la page de l'exercice
     * - reponse : réponse à l'exercice
     * - image : fichier d'image associé à la réponse à l'exercice
     * - note : note donnée par le tuteur
     *
     * POUR UN CHANGEMENT DE MOT DE PASSE :
     * - nouveaumdp : nouveau mot de passe, pour changer de mot de passe
     * - ancienmdp : ancien mot de passe
     * - nouveaumdpbis : recopie du nouveau mot de passe par l'utilisateur (doit être égal à nouveaumdp)
     */
    public void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        if (req.getCharacterEncoding() == null)
            req.setCharacterEncoding("UTF-8");
        
        // lecture des paramètres
        // on utilise apache fileupload pour pouvoir récupérer les images des exercices
        Map<String,String> parametres; // String -> String
        Map<String,FileItem> fichiers; // String -> FileItem
        final boolean isMultipart = ServletFileUpload.isMultipartContent(req); // deprecated but necessary
        if (isMultipart) {
            parametres = new Hashtable<String,String>();
            fichiers = new Hashtable<String,FileItem>();
            DiskFileItemFactory factory = new DiskFileItemFactory();
            factory.setSizeThreshold(1024*1024); // 1 Mo maximum
            ServletFileUpload upload = new ServletFileUpload(factory);
            if (req.getCharacterEncoding() == null)
                upload.setHeaderEncoding("UTF-8"); // mieux que MacRoman ! (pour éviter un bug avec les noms de fichiers)
            upload.setSizeMax(1024*1024); // on met tout en mémoire, rien sur le disque
            List items;
            try {
                items = upload.parseRequest(req); // liste de FileItem
            } catch (FileUploadException ex) {
                log("FileUploadException: " + ex.getMessage());
                envoyerErreur(resp, Strings.get("erreur_verifier_taille"));
                return;
            }
            final Iterator iter = items.iterator();
            while (iter.hasNext()) {
                FileItem item = (FileItem) iter.next();
                if (item.isFormField())
                    parametres.put(item.getFieldName(), item.getString("UTF-8"));
                else
                    fichiers.put(item.getFieldName(), item);
            }
        } else {
            parametres = new Hashtable<String,String>();
            final Iterator iter = req.getParameterMap().entrySet().iterator();
            while (iter.hasNext()) {
                Map.Entry entry = (Map.Entry)iter.next();
                String[] valeurs = (String[])entry.getValue();
                parametres.put((String)entry.getKey(), valeurs[0]);
            }
            fichiers = null;
        }
        
        // exécution des requêtes
        if (parametres.get("login") != null)
            login(req, resp, parametres);
        else if (parametres.get("titreexo") != null || parametres.get("numexercice") != null)
            reponseExercice(req, resp, parametres, fichiers);
        else if (parametres.get("nouveaumdp") != null) // changement mot de passe
            nouveauMdp(req, resp, parametres);
        else
            envoyerErreur(resp, "requête non comprise");
    }
    
    /**
     * Lecture d'un fichier sur le site par un utilisateur authentifié.
     * Aucun paramètre à la requête, seul le chemin permet de déterminer le fichier à envoyer.
     * Le chemin "/derniere-page" permet d'arriver directement à la dernière page lue.
     * Le chemin "/logout" permet de se déconnecter.
     * Le chemin "/changementmdp" permet de changer de mot de passe
     *
     * POUR UNE IMAGE D'EQUATION :
     * - equation : texte de l'équation avec la syntaxe de Jaxe
     */
    public void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        if (req.getParameter("equation") != null) {
            imageEquation(req, resp, req.getParameter("equation"));
            return;
        } else if (req.getPathInfo() != null) {
            final String codereq = req.getPathInfo();
            final String agent = req.getHeader("User-Agent");
            /*
            if (agent != null && (agent.contains("Java") || agent.contains("Cortado"))) {
                // l'appel d'applets n'envoit pas de cookie avec sessionid: session est null
                // avec les navigateurs / MacOS X (les cookies sont bien envoyés avec IE/Windows)
                // voir http://developer.apple.com/qa/qa2001/qa1124.html
                // correction : http://developer.apple.com/qa/qa2001/qa1265.html
                //
                // en plus, les applets ne connaissent pas le sessionid et donc ne peuvent ouvrir de fichier en s'identifiant
                // -> on autorise l'envoi sans identification ni log
                // (il faudrait donner l'identifiant en paramètre et gérer ça dans les applets)
                final String[] lext = {"jar", "class", "mng", "png", "gif", "jpg", "xml", "xsd", "carte", "fits", "mpg", "ogg", "ogv", "txt"};
                for (String ext : lext)
                    if (codereq.endsWith("." + ext)) {
                        envoyerPage(req, resp, codereq, baseSiteEtudiants, false);
                        // on utilise baseSiteEtudiants => il faut que les 2 sites soient synchro
                        return;
                    }
            }
            */
            final HttpSession session = req.getSession(false);
            if (session != null) {
                final Connection conn = (Connection)session.getAttribute("connection");
                if (conn != null) {
                    final User user = (User)session.getAttribute("user");
                    if (user != null) {
                        if ("/logout".equals(codereq))
                            logout(resp, session);
                        else if ("/derniere-page".equals(codereq))
                            dernierePage(resp, user);
                        else if ("/changementmdp".equals(codereq))
                            changementMdp(req, resp);
                        else
                            lecturePage(req, resp, codereq, user, conn);
                        return;
                    } else
                        log(Strings.get("erreur_user_null") + " " + codereq);
                } else
                    log(Strings.get("erreur_pas_de_connexion") + " " + codereq);
            } else {
                log(Strings.get("warning_pas_de_session") + " " + codereq);
                final boolean estPrefetch = (req.getHeader("X-moz") != null &&
                    req.getHeader("X-moz").equalsIgnoreCase("prefetch"));
                log("  " + Strings.get("url_utilisee") + " " + req.getRequestURL() + (estPrefetch ? "  ( X-moz = prefetch )" : ""));
                /* debug
                log("authType: " + req.getAuthType());
                Enumeration headernames = req.getHeaderNames();
                for (Enumeration e = headernames; e.hasMoreElements() ;) {
                    String name = (String)e.nextElement();
                    log(name + " = " + req.getHeader(name));
                }
                log("requestedSessionId: " + req.getRequestedSessionId());
                log("requestedSessionIdValid: " + req.isRequestedSessionIdValid());
                log("requestedSessionIdFromCookie: " + req.isRequestedSessionIdFromCookie());
                
                log("cookies:");
                Cookie[] cooks = req.getCookies();
                if (cooks == null)
                    log("aucun");
                else
                    for (int i=0; i<cooks.length; i++)
                        log("    " + cooks[i].getName() + " = " + cooks[i].getValue());
                */
            }
            resp.setStatus(HttpServletResponse.SC_FORBIDDEN);
            envoyerErreur(resp, Strings.get("authentification_requise_1") + " " +
                Strings.get("authentification_requise_2") +
                " <a href=\"" + pageLogin + "?redirection=" + codereq + "\">" +
                Strings.get("authentification_requise_3") + "</a>");
            return;
        }
        resp.setStatus(HttpServletResponse.SC_BAD_REQUEST);
        envoyerErreur(resp, Strings.get("requete_non_comprise"));
    }
    
    protected void login(final HttpServletRequest req, final HttpServletResponse resp, final Map<String,String> parametres) throws IOException {
        final HttpSession session = req.getSession(true);
//            session.setMaxInactiveInterval(3600); // timeout de la session: 1h
        String nom = (String)parametres.get("login");
        if (nom == null || nom.length() == 0)
            nom = "anonymous";
        final String password = (String)parametres.get("password");
        String redirection = (String)parametres.get("redirection");
        
        // recherche de l'utilisateur dans la base
        User user = null;
        final User finduser = new User(nom);
        finduser.valide = "1";
        try {
            final Vector<User> v = userDB.findUsers(finduser);
            if (v.size() > 0)
                user = v.get(0); // on prend le premier utilisateur avec le username
        } catch (JDBException ex) {
            log("JDBException: " + ex.getMessage());
        }
        if (user == null) {
            log(Strings.get("utilisateur_non_trouve") + " " + nom + " -> " + Strings.get("acces_refuse"));
            envoyerErreur(resp, Strings.get("mauvais_nom_ou_mdp"));
            return;
        }
        
        // test mot de passe
        try {
            if (password == null || !JDB.testEnc(password, user.password)) {
                log(Strings.get("mauvais_mot_de_passe") + " " + nom);
                envoyerErreur(resp, Strings.get("mauvais_nom_ou_mdp"));
                return;
            }
        } catch (NoSuchAlgorithmException ex) {
            log("login: NoSuchAlgorithmException: " + ex.getMessage());
            envoyerErreur(resp, Strings.get("acces_refuse"));
            return;
        }
        
        // test formation
        if (noms_formations != null && user.formation != null && !"système".equals(user.fonction) && !"supertuteur".equals(user.fonction) &&
                 !"tuteur".equals(user.fonction)) {
            final boolean trouve = Arrays.asList(noms_formations).contains(user.formation);
            if (!trouve) {
                log(Strings.get("mauvaise_formation") + " " + nom);
                envoyerErreur(resp, Strings.get("acces_refuse"));
                return;
            }
        }
        
        // obtention date d'aujourd'hui
        final Date now = new Date();
        final SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm");
        final String date = formatter.format(now);
        
        // test fermeture de compte
        if (user.datefermeture != null && !"".equals(user.datefermeture)) {
            Date fermeture;
            try {
                final SimpleDateFormat dfformatter = new SimpleDateFormat("yyyy-MM-dd");
                fermeture = dfformatter.parse(user.datefermeture);
                if (now.after(fermeture)) {
                    log(Strings.get("utilisation_compte_ferme") + " " + nom);
                    envoyerErreur(resp, Strings.get("acces_refuse_compte_ferme"));
                    return;
                }
            } catch (ParseException ex) {
                log("datefermeture ParseException: " + ex.getMessage());
            }
        }
        
        final Connection conn = new Connection(null, user.numero, date, null);
        try {
            connDB.addConnection(conn);
        } catch (JDBException ex) {
            log("connDB.addConnection " + ex.getMessage());
            envoyerErreur(resp, Strings.get("acces_refuse"));
            return;
        }
        // problème de concurence de threads avec 2 connections du même utilisateur démarrant en même temps
        String numconnection = null;
        try {
            Vector<Connection> v = connDB.findConnections(conn);
            if (v.size() != 0) {
                Connection conn2 = v.get(v.size()-1);
                numconnection = conn2.numero;
                conn.numero = conn2.numero;
            }
        } catch (JDBException ex) {
            log("JDBException: " + ex.getMessage());
        }
        if (numconnection == null) {
            log(Strings.get("connexion_non_trouvee") + " " + user.numero);
            envoyerErreur(resp, Strings.get("acces_refuse"));
            return;
        }
        
        session.setAttribute("user", user);
        session.setAttribute("connection", conn);
        session.setAttribute("jesuisvivant", new MyMegaBidouille(user, conn, session.getMaxInactiveInterval()));
        log("login de " + nom);
        if (redirection != null) {
            if (redirection.startsWith("Exercices")) {
                resp.sendRedirect(URLServeur + pageLogin + redirection);
                return;
            }
            if (redirection.length() > 0 && redirection.charAt(0) == '/')
                redirection = redirection.substring(1);
            String cheminSite;
            if ("étudiant".equals(user.fonction))
                cheminSite = cheminSiteEtudiants;
            else
                cheminSite = cheminSiteTuteurs;
            resp.sendRedirect(cheminSite + redirection);
        } else
            envoyerPageEntree(user, resp);
    }
    
    protected String encodageSymboles(String s) {
        // Pour contourner le bug de mySQL avec des caractères unicode sur 4 octets,
        // on vérifie que ces caractères sont sous la forme d'entités caractères en XML.
        // Avec mySQL 5.5 on pourrait utiliser utf8mb4 et on n'aurait plus besoin de cette méthode.
        // cf http://stackoverflow.com/questions/3220031/how-to-filter-or-replace-unicode-characters-that-would-take-more-than-3-bytes
        if (!s.startsWith("<?xml"))
            return(s);
        int index = 0;
        while (index < s.length()) {
            int cp = s.codePointAt(index);
            int cc = Character.charCount(cp);
            if (cp > 0xFFFF) {
                String entite = "&#" + cp + ";";
                s = s.substring(0, index) + entite + s.substring(index + cc);
                index += entite.length();
            } else {
                index += cc;
            }
        }
        return(s);
    }
    
    protected void reponseExercice(final HttpServletRequest req, final HttpServletResponse resp,
            final Map<String,String> parametres, final Map<String,FileItem> fichiers) {
        final HttpSession session = req.getSession(false);
        if (session == null) {
            log("reponseExercice: " + Strings.get("erreur_pas_de_session") + " " + req.getRequestURL());
            envoyerErreur(resp, Strings.get("session_expiree"));
            return;
        }
        final Connection conn = (Connection)session.getAttribute("connection");
        if (conn == null) {
            log("reponseExercice: " + Strings.get("erreur_pas_de_connexion_url") + " " + req.getRequestURL());
            envoyerErreur(resp, Strings.get("erreur_pas_de_connexion_identifier"));
            return;
        }
        final User user = (User)session.getAttribute("user");
        String numexercice = (String)parametres.get("numexercice");
        String titreexo = (String)parametres.get("titreexo");
        String cheminpage = (String)parametres.get("cheminpage");
        String note = (String)parametres.get("note");
        final String numuser = conn.numuser;
        if (numuser == null) {
            log("reponseExercice: " + Strings.get("erreur_pas_de_numero_utilisateur") + " " + req.getRequestURL());
            envoyerErreur(resp, Strings.get("erreur_pas_d_utilisateur_identifier"));
            return;
        }
        final SimpleDateFormat isoformatter = new SimpleDateFormat("yyyy-MM-dd");
        final String maintenant = isoformatter.format(new Date());
        final User tuteur;
        final User etudiant;
        Exercice exo;
        if ("étudiant".equals(user.fonction) || numexercice == null) {
            if (user.numtuteur != null)
                tuteur = chercherUser(user.numtuteur);
            else
                tuteur = null;
            if (tuteur == null) {
                log("reponseExercice: " + Strings.get("erreur_pas_de_tuteur") + " " + numuser);
                envoyerErreur(resp, Strings.get("erreur_attribution_tuteur"));
                return;
            }
            etudiant = user;
            exo = new Exercice();
            exo.numuser = user.numero;
            exo.chemin = cheminpage;
            exo.date = maintenant;
            exo.titre = titreexo;
            exo.reponse = encodageSymboles((String)parametres.get("reponse"));
        } else {
            try {
                exo = exoDB.getExercice(numexercice);
            } catch (JDBException ex) {
                log("Erreur interne: exoDB.getExercice " + ex.getMessage());
                exo = null;
            }
            if (exo == null) {
                log("reponseExercice: " + Strings.get("erreur_no_exercice") + " " + numexercice);
                envoyerErreur(resp, Strings.get("erreur_no_exercice"));
                return;
            }
            titreexo = exo.titre;
            cheminpage = exo.chemin;
            exo.datereponsetuteur = maintenant;
            exo.reponsetuteur = encodageSymboles((String)parametres.get("reponse"));
            if (note != null)
                exo.note = note;
            tuteur = user;
            etudiant = chercherUser(exo.numuser);
        }
        log(Strings.get("exercice") + " " + titreexo + " " + Strings.get("par") + " " + user.username);
        
        String nomImage = null;
        if (fichiers != null && fichiers.get("image") != null) {
            FileItem item = fichiers.get("image");
            if (item.isInMemory()) {
                String nomFichier = item.getName();
                if (nomFichier != null && nomFichier.indexOf('\\') != -1)
                    nomFichier = nomFichier.substring(nomFichier.lastIndexOf('\\') + 1);
                // si le nom du fichier est trop long, on coupe la fin en conservant l'extension
                if (nomFichier != null && nomFichier.length() > 40 &&
                        nomFichier.indexOf('.') == nomFichier.length() - 4)
                    nomFichier = nomFichier.substring(0, 36) + nomFichier.substring(nomFichier.length() - 4);
                nomImage = nomFichier;
                if (user == etudiant)
                    exo.nomimage = nomImage;
                else
                    exo.nomimagetuteur = nomImage;
                try {
                    if (user == etudiant)
                        exo.image = new SerialBlob(item.get());
                    else
                        exo.imagetuteur = new SerialBlob(item.get());
                } catch (SerialException ex) {
                    log("Erreur: SerialException: " + ex.getMessage());
                } catch (SQLException ex) {
                    log("Erreur: SQLException: " + ex.getMessage());
                }
            }
        }
        // enregistrement d'un nouvel exo ou modification
        if (numexercice == null) {
            try {
                exoDB.addExercice(exo);
            } catch (JDBException ex) {
                log("Erreur interne: exoDB.addExercice " + ex.getMessage());
            }
            try {
                final Vector<Exercice> vexos = exoDB.findExercices(exo);
                if (vexos.size() > 0)
                    numexercice = (vexos.get(0)).numero;
            } catch (JDBException ex) {
                log("Erreur: exoDB.findExercices " + ex.getMessage());
            }
        } else {
            try {
                exoDB.modExercice(exo);
            } catch (JDBException ex) {
                log("Erreur interne: exoDB.modExercice " + ex.getMessage());
            }
        }
        
        // envoi de l'exo au tuteur par mail, ou de la réponse du tuteur à l'étudiant
        final String from = emailAdmin;
        final String to;
        final String replyto;
        final String cheminSiteDestinataire;
        final String numetudiant;
        if (user == etudiant) {
            to = tuteur.email;
            replyto = user.email;
            cheminSiteDestinataire = cheminSiteTuteurs;
            numetudiant = null;
        } else {
            to = etudiant.email;
            replyto = tuteur.email;
            cheminSiteDestinataire = cheminSiteEtudiants;
            numetudiant = etudiant.numero;
        }
        String subject = "exercice";
        if (etudiant.formation != null && !"".equals(etudiant.formation))
            subject += " " + etudiant.formation;
        if (titreexo != null && !"".equals(titreexo))
            subject += ": " + titreexo;
        final String reponse = (String)parametres.get("reponse");
        String infoexo;
        if (user == etudiant)
            infoexo = Strings.get("nom_etudiant") + " " + etudiant.prenom + " " + etudiant.nom;
        else
            infoexo = "";
        infoexo += "\n" + Strings.get("titre_exercice") + " " + titreexo + "\n" + Strings.get("url_page") + "\n" + URLServeur;
        boolean xml = (reponse != null && reponse.startsWith("<?xml") && reponse.indexOf("<REPONSES>") != -1);
        Document doc = null;
        if (xml) {
            try {
                final DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
                final DocumentBuilder docbuilder = dbf.newDocumentBuilder();
                doc = docbuilder.parse(new InputSource(new StringReader(reponse)));
            } catch (final Exception ex) {
                log(Strings.get("envoi_mail_xml") + " " + ex.getClass().getName() + ": " + ex.getMessage());
                xml = false;
            }
        }
        if (!xml) {
            String bodyinfo = infoexo + cheminSiteEtudiants + cheminpage + "\n\n" + reponse + "\n";
            String body = infoexo + cheminSiteDestinataire + cheminpage + "\n";
            body += Strings.get("repondre_sur_site") + " " + URLServeur + pageLogin + "Exercices?numexercice=" + numexercice + "\n\n";
            body += reponse + "\n";
            
            final Mail courriel = new Mail(this, req.getServerName(), smtpServer, 25, from, to, replyto);
            courriel.definirMessage(subject, body);
            boolean avecimage = false;
            if (fichiers != null && fichiers.get("image") != null) {
                FileItem item = fichiers.get("image");
                if (item.isInMemory() && item.getSize() > 0) {
                    String nomFichier = item.getName();
                    if (nomFichier != null && nomFichier.indexOf('\\') != -1)
                        nomFichier = nomFichier.substring(nomFichier.lastIndexOf('\\') + 1);
                    ByteArrayInputStream streamContenu = new ByteArrayInputStream(item.get());
                    String mime;
                    if (nomFichier == null)
                        mime = null;
                    else
                        mime = getServletContext().getMimeType(nomFichier.toLowerCase());
                    courriel.ajouterFichierAttache(nomFichier, streamContenu, mime);
                    avecimage = true;
                }
            }
            try {
                courriel.envoyer(false);
                envoyerInfoEmail(resp, from, to, bodyinfo, avecimage, nomImage, numexercice, numetudiant);
            } catch (IOException ex) {
                log(Strings.get("envoi_mail_xml") + " " + ex.getClass().getName() + ": " + ex.getMessage());
                envoyerErreur(resp, Strings.get("erreur_envoi_mail"));
            }
            //resp.sendRedirect(baseSiteEtudiants + "exercice.html");
        } else {
            try {
                // attributs cid pour les images des équations
                final ArrayList<String> listeTexteEq = new ArrayList<String>();
                final ArrayList<String> listeCidEq = new ArrayList<String>();
                final NodeList listeEq = doc.getElementsByTagName("EQUATION");
                int nid = 1;
                for (int i=0; i<listeEq.getLength(); i++) {
                    final Element elEq = (Element)listeEq.item(i);
                    listeTexteEq.add(elEq.getAttribute("texte"));
                    final String cideq = "PAELLOS-" + nid + "-" + System.currentTimeMillis() + "@obspm.fr";
                    nid++;
                    listeCidEq.add(cideq);
                    elEq.setAttribute("cid", cideq);
                }
                
                // attributs cid pour les images des symboles
                final ArrayList<String> listeTexteSym = new ArrayList<String>();
                final ArrayList<String> listeCidSym = new ArrayList<String>();
                final NodeList listeSym = doc.getElementsByTagName("SYMBOLE");
                for (int i=0; i<listeSym.getLength(); i++) {
                    final Element elSym = (Element)listeSym.item(i);
                    final Node first = elSym.getFirstChild();
                    if (first != null) {
                        listeTexteSym.add(first.getNodeValue());
                        final String cidsym = "PAELLOS-" + nid + "-" + System.currentTimeMillis() + "@obspm.fr";
                        nid++;
                        listeCidSym.add(cidsym);
                        elSym.setAttribute("cid", cidsym);
                    }
                }
                
                /* ajouté dans le HTML plus tard
                final String infoTuteur = infoexo + cheminSiteDestinataire + cheminpage + "\n\n";
                final Node infoNode = doc.createTextNode(infoTuteur);
                final Element racine = doc.getDocumentElement();
                final Node first = racine.getFirstChild();
                if (first == null)
                    racine.appendChild(infoNode);
                else
                    racine.insertBefore(infoNode, first);
                */
                
                final Mail courriel = new Mail(this, req.getServerName(), smtpServer, 25, from, to, replyto);
                
                InputStream streamFichierAttache = null;
                String nomFichierAttache = null;
                String mimeFichierAttache = null;
                String cidFichierAttache = null;
                if (fichiers != null && fichiers.get("image") != null) {
                    FileItem item = fichiers.get("image");
                    if (item.isInMemory() && item.getSize() > 0) {
                        nomFichierAttache = item.getName();
                        if (nomFichierAttache != null && nomFichierAttache.indexOf('\\') != -1)
                            nomFichierAttache = nomFichierAttache.substring(nomFichierAttache.lastIndexOf('\\') + 1);
                        streamFichierAttache = new ByteArrayInputStream(item.get());
                        if (nomFichierAttache != null)
                            mimeFichierAttache = getServletContext().getMimeType(nomFichierAttache.toLowerCase());
                        String lf = null;
                        if (nomFichierAttache != null)
                            lf = nomFichierAttache.toLowerCase();
                        if (lf != null && (lf.endsWith(".gif") || lf.endsWith(".png") || lf.endsWith(".jpg") || lf.endsWith(".jpeg"))) {
                            cidFichierAttache = "PAELLOS-attache-" + System.currentTimeMillis() + "@obspm.fr";
                        } else {
                            // ne marchera pas avec tous les clients mail
                            courriel.ajouterFichierAttache(nomFichierAttache, streamFichierAttache, mimeFichierAttache);
                        }
                    }
                }
                
                final StringWriter sw = new StringWriter();
                reponsesXSLT.transform(new DOMSource(doc), new StreamResult(sw));
                String body = sw.toString();
                
                final String infoTuteurHTML = infoexo + cheminSiteDestinataire + cheminpage + "\n" +
                    "<a href=\"" + URLServeur + pageLogin + "Exercices?numexercice=" + numexercice +
                    "\">" + Strings.get("repondre_sur_site_html") + "</a>\n\n";
                int pdbody = body.indexOf("<body>");
                if (pdbody != -1) {
                    pdbody += 6;
                    body = body.substring(0, pdbody) + infoTuteurHTML.replace("\n","<br>") + body.substring(pdbody);
                }
                if (cidFichierAttache != null) {
                    int pfbody = body.indexOf("</body>");
                    body = body.substring(0, pfbody) + "<br><img src=\"cid:" + cidFichierAttache + "\" alt=\"" +
                        nomFichierAttache + "\">" + body.substring(pfbody);
                    /*
                    avec un document DOM on ferait qqchose comme ça :
                    final Element imgAttache = doc.createElement("img");
                    imgAttache.setAttribute("src", "cid:" + cidFichierAttache);
                    imgAttache.setAttribute("alt", nomFichierAttache);
                    racine.appendChild(doc.createElement("br"));
                    racine.appendChild(imgAttache);
                    */
                }
                
                courriel.definirMessage(subject, body, true);
                final String infoTuteurTexte = infoexo + cheminSiteDestinataire + cheminpage + "\n" +
                    Strings.get("repondre_sur_site") + " " + URLServeur + pageLogin + "Exercices?numexercice=" + numexercice + "\n\n";
                courriel.definirTexteAlternatif(infoTuteurTexte + reponse);
                
                // ajout des images des équations
                for (int i=0; i<listeTexteEq.size(); i++) {
                    final BufferedImage img = Equation.creerImage(listeTexteEq.get(i));
                    final ByteArrayOutputStream out = new ByteArrayOutputStream();
                    try {
                        Equation.enregistrerImage(img, out);
                    } catch (IOException ex) {
                        log(Strings.get("envoi_mail_xml") + " " + ex.getClass().getName() + ": " + ex.getMessage());
                        break;
                    }
                    final byte[] beq = out.toByteArray();
                    final ByteArrayInputStream streamContenu = new ByteArrayInputStream(beq);
                    courriel.ajouterFichierAttacheHTML("<" + listeCidEq.get(i) + ">", "equation"+(i+1)+".png", streamContenu, "image/png");
                }
                
                // ajout des images des symboles
                for (int i=0; i<listeTexteSym.size(); i++) {
                    final BufferedImage img = Symbole.creerImage(listeTexteSym.get(i));
                    final ByteArrayOutputStream out = new ByteArrayOutputStream();
                    try {
                        Symbole.enregistrerImage(img, out);
                    } catch (IOException ex) {
                        log(Strings.get("envoi_mail_xml") + " " + ex.getClass().getName() + ": " + ex.getMessage());
                        break;
                    }
                    final byte[] bsym = out.toByteArray();
                    final ByteArrayInputStream streamContenu = new ByteArrayInputStream(bsym);
                    courriel.ajouterFichierAttacheHTML("<" + listeCidSym.get(i) + ">", "symbole"+(i+1)+".png", streamContenu, "image/png");
                }
                
                if (cidFichierAttache != null)
                    courriel.ajouterFichierAttacheHTML("<" + cidFichierAttache + ">", nomFichierAttache, streamFichierAttache, mimeFichierAttache);
                
                courriel.envoyer(false);
                
                envoyerInfoEmail(resp, from, to, null, (nomFichierAttache != null), nomFichierAttache, numexercice, numetudiant);
            } catch (Exception ex) {
                log(Strings.get("envoi_mail_xml") + " " + ex.getClass().getName() + ": " + ex.getMessage());
                ex.printStackTrace();
                envoyerErreur(resp, Strings.get("erreur_envoi_mail"));
            }
        }
    }
    
    public void changementMdp(final HttpServletRequest req, final HttpServletResponse resp) {
        try {
            getServletContext().getRequestDispatcher("/WEB-INF/jsp/changementmdp.jsp").forward(req, resp);
        } catch (ServletException ex) {
            log("Site: changementmdp: " + ex.getClass() + ": " + ex.getMessage());
            envoyerErreur(resp, Strings.get("erreur_cf_log"));
        } catch (IOException ex) {
            log("Site: changementmdp: " + ex.getClass() + ": " + ex.getMessage());
            envoyerErreur(resp, Strings.get("erreur_cf_log"));
        }
    }
    
    protected void nouveauMdp(final HttpServletRequest req, final HttpServletResponse resp, final Map<String,String> parametres) {
        final HttpSession session = req.getSession(false);
        if (session == null) {
            log("nouveaumdp: " + Strings.get("erreur_pas_de_session") + " " + req.getRequestURL());
            envoyerErreur(resp, Strings.get("session_expiree"));
            return;
        }
        final Connection conn = (Connection)session.getAttribute("connection");
        final User user = (User)session.getAttribute("user");
        if (user != null) {
            final String ancienmdp = (String)parametres.get("ancienmdp");
            try {
                if (ancienmdp == null || !JDB.testEnc(ancienmdp, user.password)) {
                    log(Strings.get("ancien_mdp_incorrect_pour") + " " + user.username);
                    envoyerErreur(resp, Strings.get("ancien_mdp_incorrect"));
                    return;
                }
            } catch (NoSuchAlgorithmException ex) {
                log("nouveauMdp: NoSuchAlgorithmException: " + ex.getMessage());
                envoyerErreur(resp, Strings.get("erreur_changement_mdp"));
                return;
            }
            final String nouveaumdp = (String)parametres.get("nouveaumdp");
            final String nouveaumdpbis = (String)parametres.get("nouveaumdpbis");
            if (nouveaumdp != null && nouveaumdp.equals(nouveaumdpbis) && nouveaumdp.length() > 5) {
                try {
                    user.password = userDB.encryptSHA256(nouveaumdp);
                } catch (NoSuchAlgorithmException ex) {
                    log("nouveauMdp: NoSuchAlgorithmException: " + ex.getMessage());
                    user.password = userDB.encrypt(nouveaumdp);
                }
                try {
                    userDB.modUser(user);
                } catch (JDBException ex) {
                    log("userDB.modUser " + user.username + ": JDBException: " + ex.getMessage());
                    envoyerErreur(resp, Strings.get("erreur_changement_mdp"));
                }
                envoyerPageEntree(user, resp);
            } else {
                String explication = Strings.get("interrogation");
                if (nouveaumdp == null)
                    explication = Strings.get("mdp_vide");
                else if (!nouveaumdp.equals(nouveaumdpbis))
                    explication = Strings.get("mdp_differents");
                else if (nouveaumdp.length() <= 5)
                    explication = Strings.get("nouveau_mdp_trop_court");
                log(Strings.get("nouveau_mdp_incorrect_pour") + " " + user.username + " (" +
                    explication + ")");
                envoyerErreur(resp, Strings.get("nouveau_mdp_incorrect") + " (" +
                    explication + ")");
                return;
            }
        }
    }
    
    protected void lecturePage(final HttpServletRequest req, final HttpServletResponse resp, String codereq,
            final User u, final Connection conn) {
        if (codereq.endsWith("/"))
            codereq = codereq + "index.html";
        // on n'enregistre rien pour les .gif, /index.html, etc...
        final int is = codereq.lastIndexOf('/');
        final boolean estUnePage = (is != -1 && is != 0 && codereq.endsWith(".html") && !codereq.startsWith("/images/") &&
            !codereq.contains("/figure_") && !codereq.contains("/aide") && !codereq.contains("/applet") &&
            !codereq.contains("/glossaire") && !codereq.contains("/images_") && !codereq.contains("/html_images/") &&
            !codereq.contains("/media") && !codereq.contains("/impression") && !codereq.contains("/sommaire") &&
            !codereq.startsWith("/instructions-du/"));
        // on n'enregistre pas non plus si c'est un préchargement et que l'utilisateur
        // n'a pas vraiment vu la page
        final boolean estPrefetch = (req.getHeader("X-moz") != null &&
            req.getHeader("X-moz").equalsIgnoreCase("prefetch"));
        if (estUnePage)
            log("page: " + codereq + " pour " + u.username + (estPrefetch ? " (prefetch)" : ""));
        if (estUnePage && !estPrefetch) {
            if (u != null && "étudiant".equals(u.fonction)) {
                // on n'enregistre les lectures que pour les étudiants
                String alu = codereq.substring(0, is+1);
                // alu = tout le chemin, sans l'extension
                final String nomfichier = codereq.substring(is+1);
                alu += nomfichier.substring(0, nomfichier.lastIndexOf('.'));
                final String numconnection = conn.numero;
                final Lecture findlect = new Lecture(null, alu, numconnection);
                try {
                    final Vector<Lecture> v = lectDB.findLectures(findlect);
                    if (v.size() == 0) {
                        final Lecture lect = new Lecture(null, alu, numconnection);
                        lectDB.addLecture(lect);
                    }
                } catch (JDBException ex) {
                    log("Erreur: JDBException: " + ex.getMessage());
                }
            }
        }
        if (u != null && "étudiant".equals(u.fonction) && req.getRequestURI().contains(cheminSiteTuteurs)) {
            try {
                resp.sendRedirect(req.getRequestURI().replace(cheminSiteTuteurs, cheminSiteEtudiants));
            } catch (IOException ex) {
                log("lecturePage: IOException: " + ex.getMessage());
                envoyerPage(req, resp, codereq, baseSiteEtudiants, estPrefetch);
            }
        } else if (u == null || "étudiant".equals(u.fonction) || req.getRequestURI().contains(cheminSiteEtudiants))
            envoyerPage(req, resp, codereq, baseSiteEtudiants, estPrefetch);
        else
            envoyerPage(req, resp, codereq, baseSiteTuteurs, estPrefetch);
    }
    
    protected void envoyerInfoEmail(final HttpServletResponse resp, final String from, final String to, final String body,
            final boolean avecimage, final String nomImage, final String numexercice, final String numetudiant) {
        try {
            resp.setContentType("text/html; charset=UTF-8");
            final Writer out = resp.getWriter();
            final PrintWriter pw = new PrintWriter(out);
            pw.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">");
            pw.println("<html>");
            pw.println("<head>");
            pw.println("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">");
            pw.println("<title>" + Strings.get("titre_exercice") + "</title>");
            pw.println("</head>");
            pw.println("<body>");
            if (numetudiant == null)
                pw.print(Strings.get("votre_reponse_vers_tuteur"));
            else
                pw.print(Strings.get("votre_reponse_vers_etudiant"));
            pw.println(" <br><br>");
            pw.println("From: " + from + "<br>");
            pw.println("To: " + to + "<br>");
            if (body != null) {
                pw.println("<pre>");
                String bodyhtml = body;
                for (int i=0; i<bodyhtml.length(); i++) {
                    char c = bodyhtml.charAt(i);
                    if (c == '&') {
                        bodyhtml = bodyhtml.substring(0, i) + "&amp;" + bodyhtml.substring(i+1);
                        i += 4;
                    } else if (c == '<') {
                        bodyhtml = bodyhtml.substring(0, i) + "&lt;" + bodyhtml.substring(i+1);
                        i += 3;
                    } else if (c == '>') {
                        bodyhtml = bodyhtml.substring(0, i) + "&gt;" + bodyhtml.substring(i+1);
                        i += 3;
                    }
                }
                pw.println(bodyhtml);
                pw.println("</pre>");
            } else if (numexercice != null)
                pw.println("<p><a href=\"Exercices?numexercice=" + numexercice + "\">" + Strings.get("revoir_reponse") + "</a></p>");
            
            if (avecimage && numexercice != null) {
                String lf = null;
                if (nomImage != null)
                    lf = nomImage.toLowerCase();
                if (lf != null && (lf.endsWith(".gif") || lf.endsWith(".png") ||
                        lf.endsWith(".jpg") || lf.endsWith(".jpeg"))) {
                    if (body != null) {
                        if (numetudiant == null)
                            pw.println("<p><img src=\"Exercices?imageexercice=" + numexercice + "\"></p>");
                        else
                            pw.println("<p><img src=\"Exercices?imagetuteur=" + numexercice + "\"></p>");
                    }
                } else
                    pw.println("<p><strong>" + Strings.get("type_fichier_incorrect") + " \"" + nomImage +
                        "\" " + Strings.get("liste_formats") + "</strong></p>");
            }
            pw.println("<br>");
            if (numetudiant == null)
                pw.println(Strings.get("bouton_retour"));
            else
                pw.println("<p><a href=\"Exercices?numetudiant=" + numetudiant + "\">" + Strings.get("revenir_liste_exos") + "</a></p>");
            pw.println("</body>");
            pw.println("</html>");
            pw.flush();
        } catch (IOException ex) {
            log("envoyerErreur: " + ex.getMessage());
        }
    }

    protected User chercherUser(final String numuser) {
        if (numuser != null) {
            final User finduser = new User();
            finduser.numero = numuser;
            finduser.valide = "1";
            try {
                final Vector<User> v = userDB.findUsers(finduser);
                if (v.size() != 0) {
                    User tuteur = v.get(0);
                    return(tuteur);
                }
            } catch (JDBException ex) {
                log("chercherUser: JDBException: " + ex.getMessage());
            }
        }
        log("chercherUser: pas d'utilisateur " + numuser);
        return(null);
    }
    
/*
    public void sessionCreated(HttpSessionEvent se) {
        log("sessionCreated");
    }
    
    public void sessionDestroyed(HttpSessionEvent se) {
        log("sessionDestroyed");
*/ // ça ne marche pas, on ne peut rien faire dans sessionDestroyed parce-que la session est déjà détruite !
// -> bidouille avec HttpSessionBindingListener
// remarque : apparemment le comportement de sessionDestroyed est modifié à partir de Servlet 2.4

    class MyMegaBidouille implements HttpSessionBindingListener {
        User user;
        Connection conn;
        Date datefin;
        int timeout; // timeout des sessions en s
        public MyMegaBidouille(final User user, final Connection conn, final int timeout) {
            this.user = user;
            this.conn = conn;
            datefin = null;
            this.timeout = timeout;
        }
        public void setDateFin(Date d) {
            datefin = d;
        }
        public void valueBound(HttpSessionBindingEvent event) {
            //log("valueBound " + event.getName());
        }
        
        public void valueUnbound(HttpSessionBindingEvent event) {
            //log("valueUnbound " + event.getName());
            if ("jesuisvivant".equals(event.getName())) {
                SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm");
                if (datefin == null) {
                    final long temps = (new Date()).getTime(); // heure actuelle en ms
                    datefin = new Date(temps - timeout*1000);
                    if (conn.debut != null) {
                        try {
                            final Date datedebut = formatter.parse(conn.debut);
                            if (datedebut.after(datefin)) // si déconnexion causée par relancement de l'application
                                datefin = new Date(temps);
                        } catch (ParseException ex) {
                        }
                    }
                }
                String date = formatter.format(datefin);
                conn.fin = date;
                log(Strings.get("logout_de") + " " + user.username);
                try {
                    connDB.modConnection(conn);
                } catch (JDBException ex) {
                    log("Erreur: connDB.modConnection " + ex.getMessage());
                }
            }
        }
    }
    
    protected void envoyerErreur(final HttpServletResponse resp, final String errmsg) {
        try {
            resp.setHeader("Pragma", "no-cache");
            resp.setHeader("Cache-Control", "no-cache");
            resp.setHeader("Expires", "0");
            resp.setContentType("text/html; charset=UTF-8");
            final PrintWriter pw = resp.getWriter();
            pw.println("<html>");
            pw.println("<head>");
            pw.println("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">");
            pw.println("<title>" + Strings.get("titre_erreur") + "</title>");
            pw.println("</head>");
            pw.println("<body>");
            pw.println(Strings.get("une_erreur") + " " + errmsg);
            pw.println("<br>");
            pw.println(Strings.get("bouton_retour"));
            pw.println("</body>");
            pw.println("</html>");
            pw.flush();
        } catch (IOException ex) {
            log("envoyerErreur: " + ex.getMessage());
        }
    }
    
    protected void envoyerPageEntree(final User user, final HttpServletResponse resp) {
        String cheminSite;
        if ("étudiant".equals(user.fonction))
            cheminSite = cheminSiteEtudiants;
        else
            cheminSite = cheminSiteTuteurs;
        try {
            resp.sendRedirect(cheminSite + "index.html");
        } catch (IOException ex) {
            log("envoyerPageEntree: " + ex.getMessage());
            envoyerErreur(resp, Strings.get("erreur_lecture_fichier"));
        }
    }
    
    // renvoit la dernière connexion avec au moins une lecture
    private Connection derniereConnexion(final String numuser) {
        final SimpleDateFormat isoformatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm");
        Vector<Connection> connections;
        final Connection findconn = new Connection(null, numuser, null, null);
        try {
            connections = connDB.findConnections(findconn);
        } catch (JDBException ex) {
            log("Erreur: derniereConnexion: JDBException: " + ex.getMessage());
            return(null);
        }
        Date derniereDate = null;
        Connection derniereConnection = null;
        for (int ic=connections.size()-1; ic>=0; ic--) {
// la dernière connexion a plus de chances d'être en dernier, on évite ainsi trop de lectDB.findLectures
            Connection conn = connections.get(ic);
            Date debut = null;
            if (conn.debut != null) {
                try {
                    debut = isoformatter.parse(conn.debut);
                } catch (ParseException ex) {
                }
            }
            if (debut != null)
                if (derniereDate == null || debut.after(derniereDate)) {
                    Lecture lect = new Lecture(null, null, conn.numero);
                    try {
                        final Vector<Lecture> lectures = lectDB.findLectures(lect);
                        if (lectures.size() > 0) {
                            derniereDate = debut;
                            derniereConnection = conn;
                        }
                    } catch (JDBException ex) {
                        log("derniereConnexion: " + ex.getMessage());
                        return(null);
                    }
                }
        }
        return(derniereConnection);
    }
    
    protected void logout(final HttpServletResponse resp, final HttpSession session) {
        final MyMegaBidouille mmb = (MyMegaBidouille)session.getAttribute("jesuisvivant");
        if (mmb != null)
            mmb.setDateFin(new Date());
        else
            log("erreur: pas d'attribut jesuisvivant pour le logout");
        session.invalidate();
        try {
            resp.sendRedirect("../");
        } catch (IOException ex) {
            log("logout: " + ex.getMessage());
            envoyerErreur(resp, Strings.get("erreur_redirection"));
        }
    }
    
    protected void dernierePage(final HttpServletResponse resp, final User user) {
        final Connection dc = derniereConnexion(user.numero);
        if (dc == null) {
            log("dernierePage: dc == null");
            envoyerErreur(resp, Strings.get("derniere_connexion_introuvable"));
            return;
        }
        final Lecture findlect = new Lecture(null, null, dc.numero);
        Vector<Lecture> lectures;
        try {
            lectures = lectDB.findLectures(findlect);
        } catch (JDBException ex) {
            log("dernierePage: " + ex.getMessage());
            envoyerErreur(resp, Strings.get("derniere_lecture_introuvable"));
            return;
        }
        if (lectures.size() == 0) {
            log("dernierePage: lectures.size() == 0");
            envoyerErreur(resp, Strings.get("derniere_lecture_introuvable"));
            return;
        }
        // la liste de lectures n'est pas forcément dans l'ordre, il faut chercher celle
        // avec le plus grand numéro
        Lecture derniere = null;
        for (Lecture lect : lectures)
            if (derniere == null || derniere.numero.compareTo(lect.numero) < 0)
                derniere = lect;
        String redirection = derniere.alu + ".html";
        if (redirection.length() > 0 && redirection.charAt(0) == '/')
            redirection = redirection.substring(1);
        resp.setHeader("Pragma", "no-cache");
        resp.setHeader("Cache-Control", "no-cache");
        resp.setHeader("Expires", "0");
        String cheminSite;
        if ("étudiant".equals(user.fonction))
            cheminSite = cheminSiteEtudiants;
        else
            cheminSite = cheminSiteTuteurs;
        try {
            resp.sendRedirect(cheminSite + redirection);
        } catch (IOException ex) {
            log("dernierePage: " + ex.getMessage());
            envoyerErreur(resp, Strings.get("erreur_redirection"));
        }
    }
    
    protected void envoyerPage(final HttpServletRequest req, final HttpServletResponse resp,
            final String codereq, final String baseSite, final boolean prefetch) {
        try {
            final ServletContext contexte = getServletContext();
            if (contexte == null)
                throw new IOException("envoyerPage: getServletContext est null");
            final String mime = contexte.getMimeType(codereq.toLowerCase());
            if (mime == null) {
                resp.setStatus(HttpServletResponse.SC_NOT_FOUND);
                throw new IOException("envoyerPage: " + Strings.get("mime_inconnu") + " " + codereq);
            }
            final File inputFile = new File(baseSite + codereq);
            if (!inputFile.exists()) {
                resp.setStatus(HttpServletResponse.SC_NOT_FOUND);
                throw new IOException(Strings.get("fichier_inexistant") + " " + inputFile.getPath());
            }
            
            // pour mieux utiliser le cache des navigateurs
            final long lastModified = inputFile.lastModified();
            final String eTag = codereq + "_" + inputFile.length() + "_" + lastModified;
            
            final String ifNoneMatch = req.getHeader("If-None-Match");
            if (ifNoneMatch != null && matches(ifNoneMatch, eTag)) {
                resp.setHeader("ETag", eTag); // Required in 304.
                resp.sendError(HttpServletResponse.SC_NOT_MODIFIED);
                return;
            }
            
            final long ifModifiedSince = req.getDateHeader("If-Modified-Since");
            if (ifNoneMatch == null && ifModifiedSince != -1 && ifModifiedSince + 1000 > lastModified) {
                resp.setHeader("ETag", eTag); // Required in 304.
                resp.sendError(HttpServletResponse.SC_NOT_MODIFIED);
                return;
            }
            
            final String ifMatch = req.getHeader("If-Match");
            if (ifMatch != null && !matches(ifMatch, eTag)) {
                resp.sendError(HttpServletResponse.SC_PRECONDITION_FAILED);
                return;
            }
            
            final long ifUnmodifiedSince = req.getDateHeader("If-Unmodified-Since");
            if (ifUnmodifiedSince != -1 && ifUnmodifiedSince + 1000 <= lastModified) {
                resp.sendError(HttpServletResponse.SC_PRECONDITION_FAILED);
                return;
            }
            
            resp.setHeader("Accept-Ranges", "bytes");
            resp.setHeader("ETag", eTag);
            
            // on suppose les contenus statiques, en encourageant un cache pendant une journée (sauf avec prefetch)
            if (!prefetch) {
                final long maintenant = (new Date()).getTime();
                final long expiration = maintenant + 86400000;
                resp.addDateHeader("Expires", expiration);
                long derniereModif;
                try {
                    derniereModif = inputFile.lastModified();
                } catch (SecurityException ex) {
                    derniereModif = 0;
                }
                if (derniereModif != 0)
                    resp.addDateHeader("Last-Modified", derniereModif);
                else
                    resp.addDateHeader("Last-Modified", maintenant);
                resp.setHeader("Cache-Control", "private, max-age=86400"); // pour les proxy: c'est privé ! nécessite HTTP 1.1
            } else
                resp.setHeader("Cache-Control", "private");
            
            final String range = req.getHeader("Range");
            if (range == null) {
                resp.setContentType(mime);
                resp.setContentLength((int)inputFile.length());
                final OutputStream out = resp.getOutputStream();
                envoyerFichier(out, inputFile);
            } else {
                envoyerPartiesFichier(req, resp, range, inputFile, eTag, mime, true);
            }
        } catch (IOException ex) {
            log("envoyerPage: IOException: " + ex.getMessage());
            envoyerErreur(resp, Strings.get("erreur_lecture_fichier"));
        }
    }
    
    private static boolean matches(final String matchHeader, final String toMatch) {
        final String[] matchValues = matchHeader.split("\\s*,\\s*");
        Arrays.sort(matchValues);
        return Arrays.binarySearch(matchValues, toMatch) > -1
            || Arrays.binarySearch(matchValues, "*") > -1;
    }
    
    protected void envoyerFichier(final OutputStream outs, final File inputFile) {
        try {
            final BufferedInputStream bin = new BufferedInputStream(new FileInputStream(inputFile));
            final BufferedOutputStream bout = new BufferedOutputStream(outs);
            
            int length = 1024;
            final byte[] ch = new byte[length]; 
            while ((length = bin.read(ch)) != -1)
                bout.write(ch, 0, length); 
            
            bin.close();
            bout.flush();
        } catch (IOException ex) {
            log("envoyerFichier '" + inputFile.getName() + "': IOException: " + ex.getMessage());
            //envoyerErreur(resp, "Une erreur s'est produite lors de la lecture du fichier");
        }
    }
    
    protected void envoyerPartiesFichier(final HttpServletRequest req, final HttpServletResponse resp,
            final String range, final File inputFile, final String eTag, final String mime, final boolean contenu) {
        
        // merci à BalusC pour ce code
        // le paramètre contenu pourrait être utilisé pour gérer plus efficacement les requêtes HEAD
        final long length = inputFile.length();
        final long lastModified = inputFile.lastModified();
        final List<Range> ranges = new ArrayList<Range>();
        final Range full = new Range(0, length - 1, length);
        final String MULTIPART_BOUNDARY = "MULTIPART_BYTERANGES";
        
        try {
            if (!range.matches("^bytes=\\d*-\\d*(,\\d*-\\d*)*$")) {
                resp.setHeader("Content-Range", "bytes */" + length);
                resp.sendError(HttpServletResponse.SC_REQUESTED_RANGE_NOT_SATISFIABLE);
                return;
            }
            
            final String ifRange = req.getHeader("If-Range");
            if (ifRange != null && !ifRange.equals(eTag)) {
                try {
                    long ifRangeTime = req.getDateHeader("If-Range");
                    if (ifRangeTime != -1 && ifRangeTime + 1000 < lastModified) {
                        ranges.add(full);
                    }
                } catch (IllegalArgumentException ex) {
                    ranges.add(full);
                }
            }
            
            if (ranges.isEmpty()) {
                for (String part : range.substring(6).split(",")) {
                    long start = sublong(part, 0, part.indexOf("-"));
                    long end = sublong(part, part.indexOf("-") + 1, part.length());
                    
                    if (start == -1) {
                        start = length - end;
                        end = length - 1;
                    } else if (end == -1 || end > length - 1) {
                        end = length - 1;
                    }
                    
                    if (start > end) {
                        resp.setHeader("Content-Range", "bytes */" + length);
                        resp.sendError(HttpServletResponse.SC_REQUESTED_RANGE_NOT_SATISFIABLE);
                        return;
                    }
                    
                    ranges.add(new Range(start, end, length));
                }
            }
            
            RandomAccessFile input = null;
            OutputStream output = null;
            
            try {
                input = new RandomAccessFile(inputFile, "r");
                output = resp.getOutputStream();
                
                if (ranges.isEmpty() || ranges.get(0) == full) {
                    final Range r = full;
                    resp.setContentType(mime);
                    resp.setHeader("Content-Range", "bytes " + r.start + "-" + r.end + "/" + r.total);
                    
                    if (contenu) {
                        resp.setHeader("Content-Length", String.valueOf(r.length));
                        envoyerUnePartie(input, output, r.start, r.length);
                    }
                    
                } else if (ranges.size() == 1) {
                    final Range r = ranges.get(0);
                    resp.setContentType(mime);
                    resp.setHeader("Content-Range", "bytes " + r.start + "-" + r.end + "/" + r.total);
                    resp.setHeader("Content-Length", String.valueOf(r.length));
                    resp.setStatus(HttpServletResponse.SC_PARTIAL_CONTENT);
                    
                    if (contenu)
                        envoyerUnePartie(input, output, r.start, r.length);
                    
                } else {
                    resp.setContentType("multipart/byteranges; boundary=" + MULTIPART_BOUNDARY);
                    resp.setStatus(HttpServletResponse.SC_PARTIAL_CONTENT);
                    
                    if (contenu) {
                        final PrintWriter pw = new PrintWriter(output);
                        for (Range r : ranges) {
                            pw.println();
                            pw.println("--" + MULTIPART_BOUNDARY);
                            pw.println("Content-Type: " + mime);
                            pw.println("Content-Range: bytes " + r.start + "-" + r.end + "/" + r.total);
                            envoyerUnePartie(input, output, r.start, r.length);
                        }
                        pw.println();
                        pw.println("--" + MULTIPART_BOUNDARY + "--");
                    }
                }
            } finally {
                try {
                    if (input != null)
                        input.close();
                    if (output != null)
                        output.close();
                } catch (IOException ex) {
                    // ignoré
                }
            }
        } catch (IOException ex) {
            log("envoyerPartiesFichier '" + inputFile.getName() + "': " + ex.getClass().getName() + ": " + ex.getMessage());
            if (ex.getCause() != null)
                log("(" + ex.getCause().getClass().getName() + ": " + ex.getCause().getMessage() + ")");
        }
    }
    
    protected class Range {
        long start;
        long end;
        long length;
        long total;
        public Range(long start, long end, long total) {
            this.start = start;
            this.end = end;
            this.length = end - start + 1;
            this.total = total;
        }
    }
    
    private static long sublong(final String value, final int beginIndex, final int endIndex) {
        final String substring = value.substring(beginIndex, endIndex);
        return (substring.length() > 0) ? Long.parseLong(substring) : -1;
    }
    
    private static void envoyerUnePartie(final RandomAccessFile input, final OutputStream output,
            final long start, final long length) throws IOException {
        final byte[] buffer = new byte[1024];
        int read;
        
        if (input.length() == length) {
            while ((read = input.read(buffer)) > 0) {
                output.write(buffer, 0, read);
            }
        } else {
            input.seek(start);
            long toRead = length;
            
            while ((read = input.read(buffer)) > 0) {
                if ((toRead -= read) > 0) {
                    output.write(buffer, 0, read);
                } else {
                    output.write(buffer, 0, (int) toRead + read);
                    break;
                }
            }
        }
    }
    
    protected void imageEquation(final HttpServletRequest req, final HttpServletResponse resp, final String texte) {
        final HttpSession session = req.getSession(false);
        if (session == null) {
            log("exo: " + Strings.get("erreur_pas_de_session") + " " + req.getRequestURL());
            envoyerErreur(resp, Strings.get("session_expiree"));
            return;
        }
        /* pas nécessaire, et conn est inconnue dans le cas d'une connexion via exercices.html
        final Connection conn = (Connection)session.getAttribute("connection");
        if (conn == null) {
            log("reponseExercice: " + Strings.get("erreur_pas_de_connexion_url") + " " + req.getRequestURL());
            envoyerErreur(resp, Strings.get("erreur_pas_de_connexion_identifier"));
            return;
        }
        */
        final BufferedImage img = Equation.creerImage(texte);
        final ByteArrayOutputStream bout = new ByteArrayOutputStream();
        try {
            Equation.enregistrerImage(img, bout);
        } catch (IOException ex) {
            envoyerErreur(resp, Strings.get("erreur_creation_image") + " " + texte);
            log("imageEquation: " + ex.getClass().getName() + ": " + ex.getMessage() + " (" + Strings.get("equation") + " " + texte + ")");
            return;
        }
        final byte[] beq = bout.toByteArray();
        resp.setContentType("image/png");
        resp.setContentLength(beq.length);
        try {
            final BufferedOutputStream out = new BufferedOutputStream(resp.getOutputStream());
            for (int i=0; i<beq.length; i++)
                out.write(beq[i]);
            out.flush();
        } catch (IOException ex) {
            log("imageEquation: IOException: " + ex.getMessage());
        }
    }
    
}

