/*
PAELLOS - Plateforme d'Apprentissage En Ligne Libre Orientée Sémantique

Copyright (C) 2011 Observatoire de Paris

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package paellos;

import java.io.*;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.helpers.DefaultHandler;

/** Parser pour les documents XPAGES complets sans référence, contenant un site entier */

public class XPAGESParser extends DefaultHandler {
    IndexXPAGES objetIndex;
    File toutlecours;
    String labelXPAGES, nom, titrepage, contenu;
    StringBuffer texte;
    boolean dansTexte;

    public XPAGESParser(final IndexXPAGES objetIndex, final File toutlecours) {
        this.objetIndex = objetIndex; // pour appeler ajouterDocument
        this.toutlecours = toutlecours;
    }
    
    public void parse() throws IOException, SAXException {
        XMLReader xr = XMLReaderFactory.createXMLReader();
        xr.setContentHandler(this);
        //xr.setErrorHandler(this);

        xr.parse(new InputSource(new InputStreamReader(new FileInputStream(toutlecours), "ISO-8859-1")));
    }
    
    public void startDocument() {
    }

    public void endDocument() {
    }

    public void startElement(String uri, String name, String qName, Attributes atts) {
        if ("XPAGES".equals(name)) {
            labelXPAGES = atts.getValue("labelfichiers");
        } else if ("LABEL".equals(name)) {
            dansTexte = true;
            texte = null;
        } else if ("PAGE".equals(name)) {
            nom = "site/pages_" + labelXPAGES + "/" + atts.getValue("label") + ".html";
            titrepage = atts.getValue("titre");
            dansTexte = true;
            contenu = null;
            texte = null;
        } else if (dansTexte && atts.getValue("titre") != null) { // titre de SECTION, ENVIMAGE, TABLEAU, APPLET, EXERCICE, QCM
            if (texte == null)
                texte = new StringBuffer();
            else
                texte.append(' ');
            texte.append(atts.getValue("titre"));
        } else if (dansTexte && atts.getValue("mot") != null) { // pour GLOSSAIRE
            if (texte == null)
                texte = new StringBuffer();
            else
                texte.append(' ');
            texte.append(atts.getValue("mot"));
        } else if ("COMMENTAIRE".equals(name) || "SOLUTION".equals(name) || "EQUATION".equals(name) || "EQUATEX".equals(name))
            dansTexte = false; // on n'indexe pas les commentaires et solutions, et le contenu des éléments équations est du base64
    }

    public void endElement(String uri, String name, String qName) {
        if ("LABEL".equals(name)) {
            if (labelXPAGES == null || "".equals(labelXPAGES))
                labelXPAGES = new String(texte); // normalement inutile, labelfichiers doit toujours être renseigné
            dansTexte = false;
        } else if ("PAGE".equals(name)) {
            if (texte != null)
                contenu = new String(texte);
            objetIndex.ajouterDocument(nom, titrepage, contenu);
            dansTexte = false;
        } else if ("COMMENTAIRE".equals(name) || "SOLUTION".equals(name) || "EQUATION".equals(name) || "EQUATEX".equals(name))
            dansTexte = true;
    }

    public void characters(char ch[], int start, int length) {
        if (dansTexte) {
            if (texte == null)
                texte = new StringBuffer();
            else
                texte.append(' ');
            texte.append(ch, start, length);
        }
    }
}
