/*
PAELLOS - Plateforme d'Apprentissage En Ligne Libre Orientée Sémantique

Copyright (C) 2011 Observatoire de Paris

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package paellos.base;

import java.util.Vector;

import paellos.jdb.*;

public class ConnectionDB extends JDB<Connection> {
    protected String tableName = "connections";
    protected String[] fields = {"numero", "numuser", "debut", "fin"};
    protected String keyField = "numero";
    protected String createSQL =
        "create table connections (numero INT NOT NULL AUTO_INCREMENT, numuser INT, debut VARCHAR(20), " +
        "fin VARCHAR(20), PRIMARY KEY (numero)) DEFAULT CHARSET=UTF8;";
    
    public ConnectionDB(final String userDBURL, final String userDBUSER, final String userDBPASSWORD) {
        init(userDBURL, userDBUSER, userDBPASSWORD, tableName, fields, keyField, createSQL);
    }
    
    public Connection newRecord(final Object[] values) {
        return(new Connection(values));
    }
    
    public void addConnection(final Connection conn) throws JDBException {
        addRecord(conn);
    }
    
    public void modConnection(final Connection conn) throws JDBException {
        modRecord(conn);
    }
    
    public void delConnection(final Connection conn) throws JDBException {
        delRecord(conn);
    }
    
    public Connection getConnection(final String numero) throws JDBException {
        return(getRecord(numero));
    }
    
    public Vector<Connection> getAllConnections() throws JDBException {
        return(getAllRecords());
    }
    
    public Vector<Connection> findConnections(final Connection conn) throws JDBException {
        return(findRecords(conn));
    }
    
    /**
     * For debug with the command line.
     * Only the getconnection command is supported so far.
     */
    /*public static void main(String[] args) {
        if (args.length < 2) {
            printUsage();
            System.exit(1);
        }
        if (args[0].equals("getconnection")) {
            try {
                ConnectionDB connDB = new ConnectionDB();
                Connection conn = connDB.getConnection(args[1]);
                String[] values = conn.getFieldValues();
                for (int i=0; i<connDB.fields.length; i++)
                    System.out.println(connDB.fields[i] + ": " + values[i]);
            } catch (JDBException ex) {
                System.err.println("JDBException: " + ex.getMessage());
                System.exit(1);
            }
        } else {
            printUsage();
            System.exit(1);
        }
    }
    
    public static void printUsage() {
        System.out.println("usage:");
        System.out.println("java ConnectionDB " +
            "getconnection numero");
    }*/
}
