/*
PAELLOS - Plateforme d'Apprentissage En Ligne Libre Orientée Sémantique

Copyright (C) 2011 Observatoire de Paris

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package paellos.base;

import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Vector;

import paellos.jdb.*;

public class ExerciceDB extends JDB<Exercice> {
    protected String tableName = "exercices";
    protected String[] fields = {"numero", "numuser", "chemin", "date", "titre", "reponse",
        "nomimage", "image", "note", "datereponsetuteur", "reponsetuteur", "nomimagetuteur", "imagetuteur"};
    protected String keyField = "numero";
    protected String createSQL =
        "create table exercices (numero INT NOT NULL AUTO_INCREMENT, numuser INT, " +
        "chemin VARCHAR(100), date VARCHAR(15), titre VARCHAR(100), reponse TEXT, " +
        "nomimage VARCHAR(40), image MEDIUMBLOB, note VARCHAR(10), " +
        "datereponsetuteur VARCHAR(15), reponsetuteur TEXT, nomimagetuteur VARCHAR(40), imagetuteur MEDIUMBLOB, " +
        "PRIMARY KEY (numero) ) DEFAULT CHARSET=UTF8;";
    
    public ExerciceDB(final String userDBURL, final String userDBUSER, final String userDBPASSWORD) {
        init(userDBURL, userDBUSER, userDBPASSWORD, tableName, fields, keyField, createSQL);
    }
    
    public Exercice newRecord(final Object[] values) {
        return(new Exercice(values));
    }
    
    public void addExercice(final Exercice exo) throws JDBException {
        addRecord(exo);
    }
    
    public void modExercice(final Exercice exo) throws JDBException {
        modRecord(exo);
    }
    
    public void delExercice(final Exercice exo) throws JDBException {
        delRecord(exo);
    }
    
    public Exercice getExercice(final String numero) throws JDBException {
        return(getRecord(numero));
    }
    
    public Vector<Exercice> getAllExercices() throws JDBException {
        return(getAllRecords());
    }
    
    public Vector<Exercice> findExercices(final Exercice exo) throws JDBException {
        return(findRecords(exo));
    }
    
    /**
     * Renvoie la date du dernier exercice envoyé par l'utilisateur, ou null s'il n'y en a pas.
     * SELECT date FROM exercices WHERE numuser=xxx ORDER BY date DESC LIMIT 1;
     */
    public String dateDernierExercice(final String numuser) throws JDBException {
        final String sql = "SELECT date FROM exercices WHERE numuser=" + numuser + " ORDER BY date DESC LIMIT 1";
        String res = null;
        final java.sql.Connection conn = openConnection();
        Statement st = null;
        ResultSet rs = null;
        try {
            st = newStatement(conn);
            rs = query(sql, st);
            
            try {
                if (rs.next())
                    res = rs.getString(1);
            } catch (SQLException ex) {
                throw new JDBException("SQLException: " + ex.getMessage());
            }
        } finally {
            closeNoError(rs, st);
            closeConnection(conn);
        }
        return(res);
    }
}
