/*
PAELLOS - Plateforme d'Apprentissage En Ligne Libre Orientée Sémantique

Copyright (C) 2011 Observatoire de Paris

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package paellos.base;

import java.util.Vector;

import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;
import paellos.jdb.*;

public class LectureDB extends JDB<Lecture> {
    protected String tableName = "lectures";
    protected String[] fields = {"numero", "alu", "numconnection"};
    protected String keyField = "numero";
    protected String createSQL =
        "create table lectures (numero INT NOT NULL AUTO_INCREMENT, alu VARCHAR(100), " +
        "numconnection INT, PRIMARY KEY (numero) ) DEFAULT CHARSET=UTF8;";
    
    public LectureDB(final String userDBURL, final String userDBUSER, final String userDBPASSWORD) {
        init(userDBURL, userDBUSER, userDBPASSWORD, tableName, fields, keyField, createSQL);
    }
    
    public Lecture newRecord(final Object[] values) {
        return(new Lecture(values));
    }
    
    public void addLecture(final Lecture lect) throws JDBException {
        addRecord(lect);
    }
    
    public void modLecture(final Lecture lect) throws JDBException {
        modRecord(lect);
    }
    
    public void delLecture(final Lecture lect) throws JDBException {
        delRecord(lect);
    }
    
    public Lecture getLecture(final String numero) throws JDBException {
        return(getRecord(numero));
    }
    
    public Vector<Lecture> getAllLectures() throws JDBException {
        return(getAllRecords());
    }
    
    public Vector<Lecture> findLectures(final Lecture lect) throws JDBException {
        return(findRecords(lect));
    }
    
    /**
     * select l.alu, max(c.debut) from connections c, lectures l where c.numuser=xxx and l.numconnection=c.numero group by l.alu;
     */
    public String[][] dernieresLectures(final String numuser) throws JDBException {
        final String sql = "select l.alu, max(c.debut) from connections c, lectures l where c.numuser=" + numuser +
            " and l.numconnection=c.numero group by l.alu";
        
        final java.sql.Connection conn = openConnection();
        Statement st = null;
        ResultSet rs = null;
        try {
            st = newStatement(conn);
            rs = query(sql, st);
            
            final Vector<String[]> vres = new Vector<String[]>();
            
            try {
                while (rs.next()) {
                    final String[] ligne = new String[2];
                    for (int i=0; i<2; i++)
                        ligne[i] = rs.getString(i+1);
                    vres.add(ligne);
                }
            } catch (SQLException ex) {
                throw new JDBException("SQLException: " + ex.getMessage());
            }
            final String[][] tres = new String[vres.size()][2];
            vres.copyInto(tres);
            return(tres);
        } finally {
            closeNoError(rs, st);
            closeConnection(conn);
        }
    }
    
    /**
     * For debug with the command line.
     * Only the getlecture command is supported so far.
     */
    /*public static void main(String[] args) {
        if (args.length < 2) {
            printUsage();
            System.exit(1);
        }
        if (args[0].equals("getlecture")) {
            try {
                LectureDB lectDB = new LectureDB();
                Lecture lect = lectDB.getLecture(args[1]);
                String[] values = lect.getFieldValues();
                for (int i=0; i<lectDB.fields.length; i++)
                    System.out.println(lectDB.fields[i] + ": " + values[i]);
            } catch (JDBException ex) {
                System.err.println("JDBException: " + ex.getMessage());
                System.exit(1);
            }
        } else {
            printUsage();
            System.exit(1);
        }
    }
    
    public static void printUsage() {
        System.out.println("usage:");
        System.out.println("java iufm.lectdb.LectureDB " +
            "getlecture numero");
    }*/
}
