/*
PAELLOS - Plateforme d'Apprentissage En Ligne Libre Orientée Sémantique

Copyright (C) 2011 Observatoire de Paris

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/


package paellos.base;

import java.security.NoSuchAlgorithmException;
import java.util.Vector;

import paellos.jdb.*;

public class UserDB extends JDB<User> {    
    protected static String userTableName = "utilisateurs";
    public static String[] fields = {"numero", "username", "nom", "prenom", "password", "email",
        "fonction", "naissance", "formation", "niveau", "modules", "matiere", "classe", "dateinscription",
        "datefermeture", "numtuteur", "centre", "valide"};
    protected static String userKeyField = "numero";
    protected static String userCreateSQL =
        "create table utilisateurs (numero INT NOT NULL AUTO_INCREMENT, username VARCHAR(15), nom VARCHAR(30), " +
        "prenom VARCHAR(20), password VARCHAR(80), email VARCHAR(50), fonction VARCHAR(20), " +
        "naissance VARCHAR(15), formation VARCHAR(15), niveau VARCHAR(20), modules VARCHAR(20), matiere VARCHAR(40), " +
        "classe VARCHAR(40), dateinscription VARCHAR(16), datefermeture VARCHAR(16), numtuteur INT, " +
        "centre VARCHAR(20), valide TINYINT(1), PRIMARY KEY (numero)) DEFAULT CHARSET=UTF8;";
    
    public UserDB(final String userDBURL, final String userDBUSER, final String userDBPASSWORD) {
        init(userDBURL, userDBUSER, userDBPASSWORD, userTableName, fields, userKeyField,
            userCreateSQL);
    }
    
    public User newRecord(final Object[] values) {
        return(new User(values));
    }
    
    /**
     * Add a new user to the database.
     * No check for existence, but the username should be unique.
     */
    public void addUser(final User user) throws JDBException {
        addRecord(user);
    }
    
    /**
     * Modify a user record.
     * numero is the key, and the other fields are updated.
     */
    public void modUser(final User user) throws JDBException {
        modRecord(user);
    }
    
    /**
     * Delete a user record.
     * numero is the key, and the other fields are ignored.
     */
    public void delUser(final User user) throws JDBException {
        delRecord(user);
    }
    
    /**
     * Return a user record from a user number
     */
    public User getUser(final String numuser) throws JDBException {
        return(getRecord(numuser));
    }
    
    /**
     * Return all user records
     */
    public Vector<User> getAllUsers() throws JDBException {
        return(getAllRecords());
    }
    
    public Vector<User> findUsers(final User user) throws JDBException {
        return(findRecords(user));
    }

    /**
     * Return true if the password is correct for a given user.
     * (it should not be encrypted if encrypted passwords are used)
     */
    public boolean validPassword(final String usernum, final String password) throws JDBException {
        final User user = getUser(usernum);
        try {
            return(JDB.testEnc(password, user.password));
        } catch (NoSuchAlgorithmException ex) {
            throw new JDBException("NoSuchAlgorithmException: " + ex.getMessage());
        }
    }

    /**
     * For debug with the command line.
     * Only the getuser command is supported so far.
     */
    /*public static void main(String[] args) {
        if (args.length < 2) {
            printUsage();
            System.exit(1);
        }
        if (args[0].equals("getuser")) {
            try {
                UserDB userDB = new UserDB(userDBURL, userDBUSER, userDBPASSWORD);
                User user = userDB.getUser(args[1]);
                String[] values = user.getFieldValues();
                for (int i=0; i<userDB.fields.length; i++)
                    System.out.println(userDB.fields[i] + ": " + values[i]);
            } catch (JDBException ex) {
                System.err.println("JDBException: " + ex.getMessage());
                System.exit(1);
            }
        } else {
            printUsage();
            System.exit(1);
        }
    }
    
    public static void printUsage() {
        System.out.println("usage:");
        System.out.println("java iufm.userdb.UserDB " +
            "getuser username");
    }*/
}
